package Maccess;

use strict;
use lib ("/home/web-maccess/www/maccess/bin","/home/web-maccess/www/lib");

my $pkg="Maccess";

use My;
use MyXML;
use MyCGI;
use MySession;
use Maccess_db;
use Maccess_locale;
use Maccess_query_string;
use Maccess_formatfce;
use Output;
use Form;
use Date::Calc;
use Unicode::Map8;
use Unicode::String;

#-- definice zakladnich promenych pro komunikaci s SQL servrem
my $dbh_sys=Maccess_db->dbh_sys;
my $dbh_usr=Maccess_db->dbh_usr;

#-- Nasledujici 2 radky je treba upravit pro premisteni vyvojove verze na ostrou
my $maccess_ver="Maccess 1.9.1"; # verze Maccessu
my $maccess_webpath="/home/web-maccess/www";
# !!! set next line
my $maccess_path="/maccess"; # cesta ke stromu maccessu
my $maccess_abspath="$maccess_webpath$maccess_path";
my $maccess_bin_path="$maccess_path/bin"; #-- viz poznamky nize
my $maccess_lib_path="$maccess_path/lib"; #-- $lib_path je tu proto, ze balicek Maccess mohou pouzivat i jine externi programy, ktere jsou umisteny uplne jinde nez maccess a aby pak v nich fungovali odkazy na soubory gif, js, xsl, ..
                                       #-- vsude v Maccess.pm a maccesovskejch phtml programech dusledne pouzivej promenou $lib_path
my $maccess_power_user="root"; # uzivatel, ktery muze ukladat spec. nastaveni i u dotazu, ktere nejsou ulozeny (vhodne u ext. programu uzivajicich Maccess.pm)
my $maccess_query_name_ungroup=" ".translate("(ungroup)"); # dodatek k nazvu dotazu, ktery vznikl odgrupovanim

#-- definice nekterych klicu uzivanych ve %grid_field
my @grid_field_keys_fixed=('query_id','query_name','folder','query_id_group','query_name_class','query_type','num_q_tables','num_q_fields','query_group','limit_string','link');
my @grid_field_keys_Table=("name","as","jointype","joinwhere");
my @grid_field_keys_Field=("name","as","group","view","sort","where");

#-- Funkce, ktere vraci obsah nekterych casto pouzivanych promenych
sub maccess_ver { return $maccess_ver; }
sub maccess_path { return $maccess_path; }
sub maccess_lib_path { return $maccess_lib_path; }
sub maccess_power_user { return $maccess_power_user; }
sub maccess_query_name_ungroup { return $maccess_query_name_ungroup; }
sub maccess_host_url {
   my $maccess_host_url=$ENV{SCRIPT_URI}; $maccess_host_url=~s/^(\w*\:\/\/[^\/]+)\/.*$/$1/;
   return $maccess_host_url;
}
sub maccess_domain { # vyuziva se pri praci s cookie
   my $maccess_domain=$pkg->maccess_host_url;
   $maccess_domain=~s/^.*(\.[^\.]+\.[^\.:]+).*/$1/;
   return $maccess_domain;
}
sub dim { # definice dimezni jako je max nazev slozky, dotazu, ...
   return {
      folder_name => 200,
      query_name => 200,
      limit => 500,
      spec_field_name => 100
   }
}

#--------------- Zaznamenani ruznych nastavenich v cookies --------------------#
# pokud by toho melo byt vic, tak to zmen a do cookies uloz jen nejaky klic a zbytek uloz v ramci nejakych trvalych sessions
sub maccess_settings_create_cookies {
   my @cookies;
   my $query=MyCGI->cgisession;
   my $idcs=$query->param('_idcs');
   if($query->param("set")) {
      foreach my $name ("language","charset") {
         my $cookie=$query->cookie( -name=>$name, -value=>[$query->param($name)], -domain => $pkg->maccess_domain );
         push(@cookies,$cookie);
      }
   }
   return \@cookies;
}

#--------------- Priprava pro spusteni dotazu ze souboru -----------------------#
sub query_file_to_hash {
   my ($pkg_p, $query_id)=@_;
   my $grid_field;
   my $sth=$dbh_sys->prepare(q{
      select
         Query.*
      from
         Query
      where
         Query.query_id=?
   });
   $sth->execute($query_id);
   my $ref=$sth->fetchrow_hashref;
   $grid_field=$ref;
   # radek nize, protoze jsem nemohl v nazvech polozek pouzit slova "database" - jde o nesoulad v nazvech v tabulce a v promenych zde pouzivanych, ale snad to nevadi
   $$grid_field{folder}=$$grid_field{folder_id};
   $sth->finish;
   my $sth=$dbh_sys->prepare(q{
      select
         Query_field_attribute.*
      from
         Query_field_attribute
      where
         Query_field_attribute.query_id=?
   });
   $sth->execute($query_id);
   while(my $ref=$sth->fetchrow_hashref) {
      $$grid_field{"Field".$$ref{row}."_".$$ref{field_attribute}}=$$ref{value};
   }
   $sth->finish;
   my $sth=$dbh_sys->prepare(q{
      select
         Query_table_attribute.*
      from
         Query_table_attribute
      where
         Query_table_attribute.query_id=?
   });
   $sth->execute($query_id);
   while(my $ref=$sth->fetchrow_hashref) {
      $$grid_field{"Table".$$ref{row}."_".$$ref{table_attribute}}=$$ref{value};
   }
   $sth->finish;
   return $grid_field;
}

#--------------- Nacteni specialnich nastaveni dotazu ------------------------#
sub query_file_spec_to_hash {
   my ($pkg_p, $query_id,$spec_type,$name,$col_number)=@_;
   my %grid_field_spec;
   $name=substr($name,0,$pkg->dim->{spec_field_name}); # jmeno zkracuji na dimenzi s jakou je ulozeno v databazi
   my $sth=$dbh_sys->prepare(q{
      select
         Query_spec_attribute.field_attribute,
         Query_spec_attribute.value
      from
         Query_spec_attribute
      where
         Query_spec_attribute.query_id=? and
         Query_spec_attribute.query_type=? and
         Query_spec_attribute.field_name=? and
         Query_spec_attribute.field_number like ?
      order by
         Query_spec_attribute.field_number desc,
         Query_spec_attribute.field_attribute asc
   });
   # zkusim napred najit jmeno sloupce na zadane cisle sloupce a pokud nenaleznu zkusim to jeste jednou a jiz bez ohledu na cislo sloupce (pokud jich bude vic budou vzaty hodnoty prvniho sloupce - to diky razeni desc)
   # proste lepsi je, kdyz je jmeno sloupce jednoznacne
   foreach my $col_number ($col_number,"%") {
      $sth->execute($query_id,$spec_type,$name,"$col_number");
      my $founded=$sth->rows;
      while(my $ref=$sth->fetchrow_hashref) {
         $grid_field_spec{$$ref{field_attribute}}=$$ref{value};
      }
      $sth->finish;
      last if($founded);
   }
   return \%grid_field_spec;
}

#--------------- Nacteni specialnich nastaveni dotazu (parametry ffce) ----------#
sub query_file_spec_to_hash_ffce {
   my ($pkg_p, $query_id,$spec_type,$name,$col_number,$cols)=@_;
   my $formatfce;
   my $ffcevarcol={};
   my $grid_field_spec=$pkg->query_file_spec_to_hash($query_id,$spec_type,$name,$col_number);
   foreach my $field_attribute (keys %{$grid_field_spec}) {
      if($field_attribute eq "formatfce") {
         $formatfce=$$grid_field_spec{$field_attribute};
      }
      elsif($field_attribute=~/^ffce(var|col)(.+)$/) {
         my $varcol=$1;
         my $ffcename=$2;
         # udelano tak, ze var vzdy prevalcuje col
         if($varcol eq "var") {
            $$ffcevarcol{"ffce"."var".$ffcename}=$$grid_field_spec{$field_attribute};
            delete($$ffcevarcol{"ffce"."col".$ffcename});
         }
         else {
            if(!defined($$ffcevarcol{"ffce"."var".$ffcename})) {
               my $col_last;
               my $num_fields=@$cols;
               for(my $col=0;$col<$num_fields;$col++) {
                  if($$cols[$col]{name} eq $$grid_field_spec{$field_attribute}) {
                     $col_last=$col;
                     last;
                  }
               }
               $$ffcevarcol{"ffce"."col".$ffcename}=$col_last;
            }
         }
      }
   }
   return [$formatfce,$ffcevarcol];
}

#--------------- Samotne spusteni maccessovske externi fce ----------------------#
sub maccess_formatfce {
   my ($pkg_p, $tO,$val,$maccess_formatfce,$datarow)=@_;
   my ($formatfce,$ffcevarcol)=@{$maccess_formatfce};
   my $ffce;
   foreach my $ffcevarcolname (keys %{$ffcevarcol}) {
      if($ffcevarcolname=~/^ffce(var|col)(.+)$/) {
         my $varcol=$1;
         my $ffcename=$2;
         if($varcol eq "var") {
            $$ffce{$ffcename}=$$ffcevarcol{$ffcevarcolname};
         }
         else {
            $$ffce{$ffcename}=$$datarow[$$ffcevarcol{$ffcevarcolname}]{value};
         }
      }
   }
   Maccess_formatfce->$formatfce($tO,$val,$ffce);
}

#--------------- Priprava pro spusteni dotazu z query mrizky --------------------#
sub query_grid_to_hash {
   my ($pkg_p, $query)=@_;
   my %grid_field;
   my $idcs=$query->param('_idcs');
   foreach my $key (@grid_field_keys_fixed) {
      $grid_field{$key}=$query->param($key);
   }
   for(my $i=0;$i<$grid_field{num_q_fields};++$i) {
      foreach my $field_attribute (@grid_field_keys_Field) {
         $grid_field{"Field".$i."_".$field_attribute}=$query->param("Field".$i."_".$field_attribute);
      }
   }
   for(my $i=0;$i<$grid_field{num_q_tables};++$i) {
      foreach my $table_attribute (@grid_field_keys_Table) {
         $grid_field{"Table".$i."_".$table_attribute}=$query->param("Table".$i."_".$table_attribute);
      }
   }
   $grid_field{query_id_group}*=1; # zajistuje definovanou numerickou hodnotu
   # nacteni seznamu tabulek ktere se maji zobrazovat z session
   my $idqc=$query->param('idqc');
   my $session=MySession->read($idqc);
   $grid_field{showtables}=join(",",@{$session});
   return \%grid_field;
}

#--------------- Generace hashe navrhu poddotazu z hashe navrhu dotazu
## obdobnou vec delas ve fci Maccess_query_string->query_string_subgrp - casem by meli fce nejak scasti splynout
sub query_hash_to_ungroup_hash {
   my ($pkg_p,$grid_field)=@_;
   my %grid_field_ungroup;
   foreach my $key (@grid_field_keys_fixed) {
      $grid_field_ungroup{$key}=$$grid_field{$key};
   }
   $grid_field_ungroup{query_name}= $grid_field_ungroup{query_name} . $maccess_query_name_ungroup;
   $grid_field_ungroup{query_id_group}=$grid_field_ungroup{query_id};
   $grid_field_ungroup{query_id}=undef;
   $grid_field_ungroup{query_group}="0";
   for(my $i=0;$i<$$grid_field{num_q_tables};++$i) {
      foreach my $table_attribute (@grid_field_keys_Table) {
         $grid_field_ungroup{"Table${i}_$table_attribute"}=$$grid_field{"Table${i}_$table_attribute"};
      }
   }
   $grid_field_ungroup{"Field0_name"}="*";
   $grid_field_ungroup{"Field0_view"}="1";
   my $j=20;
   for(my $i=0;$i<$$grid_field{num_q_fields};++$i) {
      if($$grid_field{"Field${i}_group"} eq "group") {
         ++$j;
         $grid_field_ungroup{"Field${j}_name"}=$$grid_field{"Field${i}_name"};
         $grid_field_ungroup{"Field${j}_where"}= "=\"[" . $$grid_field{"Field${i}_as"} . ":]\"";
      }
      elsif($$grid_field{"Field${i}_where"}) {
         if($$grid_field{query_group} and $$grid_field{"Field${i}_view"} and ($$grid_field{"Field${i}_as"} or $$grid_field{"Field${i}_group"} ne "expr")) {  # podmiky HAVING neuvazuj
         }
         else {
            ++$j;
            $grid_field_ungroup{"Field${j}_name"}=$$grid_field{"Field${i}_name"};
            $grid_field_ungroup{"Field${j}_where"}=$$grid_field{"Field${i}_where"};
         }
      }
   }
   $grid_field_ungroup{num_q_fields}=$j+1;
   return \%grid_field_ungroup;
}

#--------------- Ulozeni dotazu do souboru -------------------------------------#
sub query_save {
   my ($pkg_p, $query)=@_;
   my $grid_field=$pkg->query_grid_to_hash($query);
   my $info_query=$pkg->info_query($$grid_field{query_id});
   # pokud se ma dotaz ulozit jako novy, nebo pokud dotaz neexistuje, bude vytvoren (bude mu prideleno pozdeji nove jedinecne id)
   if($query->param('saveas_form') or !$$info_query{_exist}) {
      $$grid_field{query_id}=undef;
   }
   ### ZMENA rozhodl jsem se umoznit duplicitu jmen dotazu v ramci jedne slozky
   ###       a proto je kod nize vypoznamkovan
   ###       (z umoznenim duplicity jmen vylozene hodlam pocitat v ramci
   ###       ukladani (pod)dotazu)
   #### pokud jmeno dotazu v dane slozce jiz existuje, jedna se o prepisovani jiz existujiciho dotazu s prislusnym query_id (spravne by tu patrila otazka, zda chi opravdu dotaz prepsat)
   ###my $info_query_name=$pkg->info_query_name($$grid_field{folder},$$grid_field{query_name});
   ###if($$info_query_name{_exist}) {
   ###   $$grid_field{query_id}=$$info_query_name{query_id};
   ###}
   # pokud se ma dotaz ulozit jako novy, je nutne zrusit pripadnou vazbu na group (nad)dodaz
   if($query->param('saveas_form')) {
      $$grid_field{query_id_group}=0
   }
   # pokud jde o ukladani ungroup (pod)dotazu, je treba zjistit nektere informace, nektere informace upravit a pozdeji jeste neco otestovat
   my $info_query_group;
   if($$grid_field{query_id_group}>0) {
      # zjisti info o (nad)dotaze
      $info_query_group=$pkg->info_query($$grid_field{query_id_group});
      # pokud (nad) dotaz neexistuje, tak zrus ukladani jako (pod)dotazu
      if(!$$info_query_group{_exist}) {
         $$grid_field{query_id_group}=0
      }
      else {
         # zajisteni sladeni folderu podle (nad)dotazu
         $$grid_field{folder}=$$info_query_group{folder_id};
         # zjisti zpetne info o (pod)dotaze
         my $info_query_ungroup=$pkg->info_query_ungroup($$info_query_group{query_id});
         # pokud id (pod)dotazu neexistuje, tak zajisti, ze jde o ukladani noveho (pod)dotazu - je to tu pro jistotu
         if(defined($$grid_field{query_id}) and !$$info_query_ungroup{_exist}) {
            $$grid_field{query_id}=undef;
         }
         # jinak zajisti spravne id - je to tu pro jistotu
         else {
            $$grid_field{query_id}=$$info_query_ungroup{query_id};
         }
      }
   }
   # dotaz nesmi ulozit nikdo jiny nez jeho tvurce ci tvurce slozky a dotaz musi mit neprazdne jmeno
   if(
      (
         defined($$grid_field{query_id}) and
         $pkg->create_user_query($$grid_field{query_id}) ne $pkg->get_remote_user and
         $pkg->create_user_folder($$grid_field{folder}) ne $pkg->get_remote_user
      ) or
      !length($$grid_field{query_name})
   ) {
      my $idcs=$query->param('_idcs');
      my $outputdev=$pkg->outputdev($query);
      my ($docO,$htmlO,$bodyO)=$pkg->creahtml(
         $outputdev,
         $idcs,
         {
            h1 => $$grid_field{query_name},
            query_id => $$grid_field{query_id},
            folder => $$grid_field{folder}
         }
      );
      if(my ($bodyT)=$htmlO->findnodes('body')) {
         bless($bodyT,"MyXML");
         $bodyT->sA("class","red");
      }
      $bodyO->aT("p", {}, translate("Query :0 can be changed only by its creator - :1 or by the folder creator - :2.",$$grid_field{query_name},$pkg->create_user_query($$grid_field{query_id}),$pkg->create_user_folder($$grid_field{folder})));
      Output->outputprint($docO,$outputdev);
   }
   # ungroup dotaz nesmi ulozit nikdo jiny nez tvurce group (nad)dotazu
   elsif(
      $$info_query_group{_exist} and
      $pkg->create_user_query($$info_query_group{query_id}) ne $pkg->get_remote_user and
      $pkg->create_user_folder($$grid_field{folder}) ne $pkg->get_remote_user
   ) {
      my $idcs=$query->param('_idcs');
      my $outputdev=$pkg->outputdev($query);
      my ($docO,$htmlO,$bodyO)=$pkg->creahtml(
         $outputdev,
         $idcs,
         {
            h1 => $$grid_field{query_name},
            query_id => $$grid_field{query_id},
            folder => $$grid_field{folder}
         }
      );
      if(my ($bodyT)=$htmlO->findnodes('body')) {
         bless($bodyT,"MyXML");
         $bodyT->sA("class","red");
      }
      $bodyO->aT("p", {}, translate("Query :0 can be changed only by group query creator - :1 or by the folder creator - :2.",$$grid_field{query_name},$pkg->create_user_query($$info_query_group{query_id}),$pkg->create_user_folder($$grid_field{folder})));
      Output->outputprint($docO,$outputdev);
   }
   # vse je ok a dotaz bude ulozen
   else {
      # ulozeni do Query
      my $sth=$dbh_sys->prepare(q{
         replace
            Query
         set
            Query.query_id=?,
            Query.query_name=?,
            Query.folder_id=?,
            Query.query_id_group=?,
            Query.query_name_class=?,
            Query.create_user=?,
            Query.query_type=?,
            Query.query_group=?,
            Query.num_q_tables=?,
            Query.num_q_fields=?,
            Query.limit_string=?,
            Query.showtables=?,
            Query.link=?
      });
      $sth->execute(
         $$grid_field{query_id},
         $$grid_field{query_name},
         $$grid_field{folder},
         $$grid_field{query_id_group},
         $$grid_field{query_name_class},
         $pkg->get_remote_user,
         $$grid_field{query_type},
         $$grid_field{query_group},
         $$grid_field{num_q_tables},
         $$grid_field{num_q_fields},
         $$grid_field{limit_string},
         $$grid_field{showtables},
         $$grid_field{link}
      );
      if(!defined($$grid_field{query_id})) {
         my $sth=$dbh_sys->prepare(q{
            select
               last_insert_id() as query_id
         });
         $sth->execute;
         my $ref=$sth->fetchrow_hashref;
         $sth->finish;
         $$grid_field{query_id}=$$ref{query_id};
      }
      # ulozeni do Query_field_attribute
      my $sth=$dbh_sys->prepare(q{
         delete from
            Query_field_attribute
         where
            Query_field_attribute.query_id=?
      });
      $sth->execute($$grid_field{query_id});
      my $sth=$dbh_sys->prepare(q{
         insert into
            Query_field_attribute
         set
            Query_field_attribute.query_id=?,
            Query_field_attribute.row=?,
            Query_field_attribute.field_attribute=?,
            Query_field_attribute.value=?
      });
      for(my $i=0;$i<$$grid_field{num_q_fields};++$i) {
         foreach my $field_attribute (@grid_field_keys_Field) {
            my $value=$$grid_field{"Field".$i."_".$field_attribute};
            $value.="";
            $sth->execute($$grid_field{query_id},$i,$field_attribute,$value);
         }
      }
      # ulozeni do Query_table_attribute
      my $sth=$dbh_sys->prepare(q{
         delete from
            Query_table_attribute
         where
            Query_table_attribute.query_id=?
      });
      $sth->execute($$grid_field{query_id});
      my $sth=$dbh_sys->prepare(q{
         insert into
            Query_table_attribute
         set
            Query_table_attribute.query_id=?,
            Query_table_attribute.row=?,
            Query_table_attribute.table_attribute=?,
            Query_table_attribute.value=?
      });
      for(my $i=0;$i<$$grid_field{num_q_tables};++$i) {
         foreach my $table_attribute (@grid_field_keys_Table) {
            my $value=$$grid_field{"Table".$i."_".$table_attribute};
            $value.="";
            $sth->execute($$grid_field{query_id},$i,$table_attribute,$value);
         }
      }
      #My->redirect("query_create.phtml?" . "query_id=" . encode_uri($$grid_field{query_id}));
      My->redirect("./?folder=" . encode_uri($$grid_field{folder}));
   }
   return $$grid_field{query_id};
}

#-----------------------------
sub to_sql {
   my ($pkg_p, $grid_field_p)=@_;
   my %grid_field=%$grid_field_p;
   my %query_hash;
   $query_hash{LIMIT}=$grid_field{limit_string};
   my $order_string_xml_group="";
   my @cols_maccess_type;
   for(my $i=0;$i<$grid_field{num_q_tables};++$i) {
      if(my $Table_name=      $grid_field{ "Table${i}_name" }) {
         my $Table_as=        $grid_field{ "Table${i}_as" };
         my $Table_jointype=  $grid_field{ "Table${i}_jointype" };
         my $Table_joinwhere= $grid_field{ "Table${i}_joinwhere" };
         if($Table_jointype) {
            $query_hash{FROM}.=" $Table_jointype ";
         }
         $query_hash{FROM}.=$Table_name;
         if($Table_as) {
            $query_hash{FROM}.=" as $Table_as";
         }
         if($Table_joinwhere) {
            $query_hash{FROM}.=" on ($Table_joinwhere)";
         }
      }
   }
   my $j=-1;
   for(my $i=0;$i<$grid_field{num_q_fields};++$i) {
      if(my $Field_name=   $grid_field{ "Field${i}_name" }) {
         my $Field_group=  $grid_field{ "Field${i}_group" };
         my $Field_as=     $pkg->encode_maccess($grid_field{ "Field${i}_as" });
         my $Field_view=   $grid_field{ "Field${i}_view" };
         my $Field_sort=   $grid_field{ "Field${i}_sort" };
         my $Field_where=  $grid_field{ "Field${i}_where" };
         if($Field_sort eq "xml_group") { #- zajisteni, ze pri $Field_sort eq 'xml_group' je vlastne jakoby vzdy zatrzeno Zobr. -> je dulezite, aby ve vystupu byl sloupec, kde ktereho je xml_group
            $Field_view=1;
         }
         if($grid_field{query_group} and !(($Field_group eq "group") or ($Field_group eq "expr"))) {
            $Field_name="$Field_group($Field_name)";
         }
         else {
            $Field_name=$grid_field{"Field${i}_name"};
         }
         if($Field_view or ($Field_group eq "group")) {
            ++$j;
            if($Field_as) {
               $query_hash{COLS}.=$Field_name." as ".$Field_as.", ";
            }
            else {
               $query_hash{COLS}.=$Field_name.", ";
            }
            #- urceni typu sloupce v maccessu
            if(!$Field_view) {
               $cols_maccess_type[$j]="hidden_group";
            }
            elsif($Field_sort eq "xml_group" and $grid_field{query_type}!~/^update/) {
               $cols_maccess_type[$j]="xml_group";
            }
            elsif($Field_sort eq "style") {
               $cols_maccess_type[$j]="style";
            }
            elsif($Field_sort eq "href") {
               $cols_maccess_type[$j]="href";
            }
            else {
               $cols_maccess_type[$j]="value";
            }

         }
         if($Field_where) {
            if($grid_field{query_group} and $Field_view and ($Field_as or $Field_group ne "expr")) { # pripadna zmena v teto podmince se musi promitnout i ve funkci query_hash_to_ungroup_hash
               $query_hash{HAVING}.=Maccess_query_string->modify_Field_where($Field_where, $Field_as || $Field_name)." and ";
            }
            else {
               $query_hash{WHERE}.="(".Maccess_query_string->modify_Field_where($Field_where,$Field_name).") and ";
            }
         }
         if($grid_field{query_group} and ($Field_group eq "group")) {
            if($Field_as) {
               $query_hash{"GROUP BY"}.=$Field_as.",";
            }
            else {
               # zmena - pozor predpokladam, ze je jedinecne pojmenovan - jinak je treba upravit query_string_subgrp
               # $query_hash{"GROUP BY"}.=$Field_name.",";
               die encode_entities(translate("Column :0 must be named in box \"as\".",$Field_name));
            }
         }
         if($Field_sort) {
            if($Field_as and ($Field_sort eq "asc" or $Field_sort eq "desc")) {
               $query_hash{"ORDER BY"}.=$Field_as." ".$Field_sort.",";
            }
            elsif($Field_sort eq "asc" or $Field_sort eq "desc") {
               $query_hash{"ORDER BY"}.=$Field_name." ".$Field_sort.",";
            }
            elsif($Field_as and $Field_sort eq "xml_group") {
               $order_string_xml_group.=$Field_as." asc,";
            }
            elsif($Field_sort eq "xml_group") {
               $order_string_xml_group.=$Field_name." asc,";
            }
         }
      }
   }
   $query_hash{COLS}=substr($query_hash{COLS},0,length($query_hash{COLS})-2); # rezu ", "
   $query_hash{WHERE}=substr($query_hash{WHERE},0,length($query_hash{WHERE})-5); # rezu " and "
   $query_hash{"GROUP BY"}=substr($query_hash{"GROUP BY"},0,length($query_hash{"GROUP BY"})-1); # rezu ","
   $query_hash{HAVING}=substr($query_hash{HAVING},0,length($query_hash{HAVING})-5); # rezu " and "
   $query_hash{"ORDER BY"}=$order_string_xml_group.$query_hash{"ORDER BY"};
   $query_hash{"ORDER BY"}=substr($query_hash{"ORDER BY"},0,length($query_hash{"ORDER BY"})-1); # rezu ","
   my $query_string=Maccess_query_string->query_hash_to_query_string(\%query_hash);
   my %param;
   $param{folder}=$grid_field{folder};
   $param{query_id}=$grid_field{query_id};
   $param{query_name}=$grid_field{query_name};
   $param{query_group}=$grid_field{query_group};
   $param{query_type}=$grid_field{query_type};
   $param{query_string}=$query_string;
   $param{link}=$grid_field{link};
   $param{cols_maccess_type}=\@cols_maccess_type;
   return (\%param);
}


#--------------- Obsahuje-li retezec "promene - [Promena:hodnota]" bude vytvoren formular -----------#
sub query_form {
   my ($pkg_p, $param,$polozky)=@_;
   my $query=MyCGI->cgisession;
   my $idcs=$query->param('_idcs');
   # Funkce prevalcuje nektere hodnoty v param, pokud prisli primo GET ci POST metodou v pozadavku
   $param=$pkg->update_param_by_GET_or_POST($param,$query);
   # zaznamenani udaju v param do session
   my $id=$pkg->sessionwrite(undef,$param);
   # tvorba html kodu
   my $outputdev=$pkg->outputdev($query);
   my ($docO,$htmlO,$bodyO)=$pkg->creahtml(
      $outputdev,
      $idcs,
      {
         h1 => $$param{query_name},
         query_id => $$param{query_id},
         folder => $$param{folder}
      }
   );
   if(
      length($$param{query_id}) and
      Maccess->create_user_query($$param{query_id}) eq Maccess->get_remote_user and
      !defined($pkg->query_id_system)
   ) {
      # zmena standadtni maccessovske hlavicky
      if(my ($headO)=$htmlO->findnodes('head')) {
         bless($headO,"MyXML");
         my $href=$pkg->href_spec_form($id);
         $headO->aT(
            "script",
            {
               type => "text/javascript"
            },
            q/
               myMenuForm[0]=new Array(5);
               myMenuForm[0][0]="/ . translate("spec. settings") . q/";
               myMenuForm[0][1]="/.$href.q/";
            /
         );
      }
   }
   # uzivatelska cast formulare plynouci z $$param{query_string}
   if(!defined($polozky)) { # $polozky jsou definovany jen pokud formular neprosel kontrolou
      $polozky=$pkg->query_form_def($param,$query);
   }
   my ($formO,$divO)=Form->form_val_list($bodyO,$polozky,[],"get");
   # neuzivatelska ciste systemove Maccessovska cast formulare
   $divO->aT(
      "input",
      {
         type => "hidden",
         name => "id",
         value => $id
      }
   );
   # link
   $divO->aT(
      "input",
      {
         type => "hidden",
         name => "link",
         value => $$param{link}
      }
   );
   # cudl (form_do - po nasledujicim submi jde o informaci, ze dotaz byl spusten z formulare)
   my $pO=$formO->aT("p");
   $pO->aT(
      "input",
      {
         name => "form_do",
         type => "submit",
         value => translate("Run")
      }
   );
   # query_type
   if(!defined($pkg->query_id_system)) {
      my $pO=$formO->aT(
         "p",
         {
            style => "font-style: italic; margin-top: 60px"
         }
      );
      $pO->aT(undef, {}, translate("Output:"));
      $pO->aT(undef, {}, " ");
      $pkg->selectT_query_type($pO,$$param{query_type});
   }
   # odeslani vystupu
   Output->outputprint($docO,$outputdev);
   # doplneni udaju do param pro pohodlne vyvolani uprav na specialni nastaveni formulare
   delete($$param{vars_maccess_spec_name});
   foreach my $name (Form->names($polozky)) {
      push(@{$$param{vars_maccess_spec_name}},{ name => $name, type => $$polozky{$name}{type} });
   }
   # zaznamenani udaju v param do session - znovu, jelikoz jsem doplnoval $$param{vars_maccess_spec_name}
   $pkg->sessionwrite($id,$param);
}

sub query_form_def {
   my ($pkg_p, $param,$query)=@_;
   my $polozky;
   ## zadna krasavice - obdaba viz tez Maccess_query_string->insert_values_to_query_string
   my @query_string_split=split(/\[([^\]:]+:[^\]]*)\]/si,$$param{query_string});
   my @variables;
   for(my $i=1;$i<=$#query_string_split;$i+=2) {
      my @variable=split(/:/si,$query_string_split[$i],2);
      if(!grep(/^\Q$variable[0]\E$/, @variables)) {
         push(@variables,$variable[0]);
         my $name="VAR_".$variable[0];
         # nacerpani parametru ze spec nastaveni pro formular
         my $grid_var_spec=$pkg->query_file_spec_to_hash($$param{query_id},"form",$name,"");
         foreach my $attr (%{$grid_var_spec}) {
            $$polozky{$name}{$attr}=$$grid_var_spec{$attr};
         }
         # doplneni dalsich parametru
         if(!defined($$polozky{$name}{print})) {
            $$polozky{$name}{print}=$variable[0];
         }
         if($variable[1]=~/^SELECT /si) {
            my $selectmenu=$pkg->selectT_menu_q($variable[1]);
            if(ref($selectmenu)) {
               $$polozky{$name}{type}="select";
               $$polozky{$name}{selectmenu}=$selectmenu;
               $$polozky{$name}{value}=$$selectmenu[0][0];
            }
            else {
               $$polozky{$name}{type}="text";
               $$polozky{$name}{value}=$selectmenu;
            }
         }
         elsif($variable[1]=~/\|/si) {
            my $selectmenu=$pkg->selectT_menu($variable[1]);
            $$polozky{$name}{type}="select";
            $$polozky{$name}{selectmenu}=$selectmenu;
            $$polozky{$name}{value}=$$selectmenu[0][0];
         }
         else {
            $$polozky{$name}{type}="text";
            $$polozky{$name}{value}=$variable[1];
         }
         push(@{$$polozky{_layout}[0][0]{names}}, ($name));
      }
   }
   # hodnoty v cookie maji prednost pred preddefinovanymi hodnotami v dotaze
   $polozky=Form->values_predef($polozky,$query);
   return $polozky;
}

#-------------- Formular zobrazujici query_string retezec pro "select" dotaz ---#
sub query_string_print {
   my ($pkg_p, $param)=@_;
   # zaznamenani udaju v param do session
   my $id=$pkg->sessionwrite(undef,$param);
   my $query=MyCGI->cgisession;
   my $idcs=$query->param('_idcs');
   my $outputdev=$pkg->outputdev($query);
   my ($docO,$htmlO,$bodyO)=$pkg->creahtml(
      $outputdev,
      $idcs,
      {
         h1 => $$param{query_name},
         query_id => $$param{query_id},
         folder => $$param{folder}
      }
   );
   my $formO=$bodyO->aT(
      "form",
      {
         name => "Query_Select",
         action => My->okruh,
         method => "post"
      }
   );
   my $pO=$formO->aT("p");
   $pO->aT(
      "input",
      {
         type => "hidden",
         name => "id",
         value => $id
      }
   );
   $pO->aT(
      "textarea",
      {
         name => "query_string",
         rows => "15",
         cols => "70"
      },
      $$param{query_string}
   );
   my $pO=$formO->aT("p");
   $pO->aT(
      "input",
      {
         name => "viewSQL_do",
         type => "submit",
         value => translate("Run")
      }
   );
   my $pO=$bodyO->aT("p");
   $pO->aT("b", {}, "SELECT syntax:");
   $bodyO->aT(
      "pre",
      {},
      '
         SELECT
            select_expression,...
            [FROM table_references
               [WHERE where_definition]
               [GROUP BY {unsigned_integer | col_name | formula}]
               [HAVING where_definition]
               [ORDER BY {unsigned_integer | col_name | formula} [ASC | DESC] ,...]
               [LIMIT [offset,] rows]
      '
   ); 
   my $pO=$bodyO->aT("p");
   $pO->aT("b", {}, "JOIN syntax:");
   $bodyO->aT(
      "pre",
      {},
      '
         table_reference INNER JOIN table_reference ON conditional_expr
         table_reference LEFT OUTER JOIN table_reference ON conditional_expr
      '
   );
   Output->outputprint($docO,$outputdev);
   return 1;
}

#--------------- Odeslani pozadavku SQL serveru a zobrazeni vysledku -----------#
sub query_to_array {
   my ($pkg_p, $param)=@_;
   my $query_string=$$param{query_string};
   #-- Jestlize neexistuje limit, pak bude pridelen - dulezite u dotazu vzniklych pri ungroup
   my ($query_string_no_limit, @limit)=Maccess_query_string->query_string_no_limit($query_string);
   if(!@limit) {
      @limit=(0,$pkg->dim->{limit});
      my $query_hash=Maccess_query_string->query_string_to_query_hash($query_string);
      $$query_hash{LIMIT}=join(",",@limit);
      $query_string=Maccess_query_string->query_hash_to_query_string($query_hash);
   }
   my @cols_maccess_type;
   if(defined($$param{cols_maccess_type})) {
      @cols_maccess_type=@{$$param{cols_maccess_type}};
   }
   # zaznamenani udaju v param do session
   my $id=$pkg->sessionwrite(undef,$param);
   #-- priprava dotazu a spusteni dotazu
   my (@data,@cols);
   my ($limit,%cross_ref,$hidden);
   my ($sth);
   $sth=$dbh_usr->prepare(Maccess_query_string->convert_some_strings($query_string));
   my $error;
   eval {
      local $SIG{__WARN__}=sub { $error=$_[0]; };
      my $rv=$sth->execute;
   };
   #---- spusteni dotazu selhalo - nejspis jde o chybnou synaxy
   if(defined($error)) {
      $pkg->print_error_syntax($param,$error);  # tiskne kompletni HTML stranku
      exit;
   }
   #---- spusteni dotazu bylo ok
   my $num_rows=$sth->rows;
   my $names=$sth->{NAME};
   my $names_type=$sth->{TYPE};
   my $num_fields=$sth->{NUM_OF_FIELDS};
   my $icon;
   #---- nejde o krizovy dotaz
   if($$param{query_group}!=2) {
      my @cols_group;
      if($$param{link} eq "no") {
         $icon=0 ;
      }
      else {
         $icon=1;
         @cols_group=Maccess_query_string->query_string_cols_group($query_string);
      }
      #- typ linku
      if($$param{query_type} eq "xml_table" and Maccess_query_string->may_be_ungroup_query($query_string) and $icon) {    ##--- $query_string=~/ group by /i - zduvodu, ze Maccess zatim umi rozbalovat jen souhrny dotaz pokud obsahuje group by
         $icon=1;
         $cols[0]{name}="+";  # ungroup
         $cols[0]{type}="";
         $cols[0]{chosen}="9";  # zajisti, ze sloupec ikon bude v class="chosen9" - a nebude videt pri tisku
      }
      elsif($$param{query_type} eq "xml_table" and $icon) { ## vyrazeno:  and Maccess_query_string->may_be_update_query($query_string) ## fce may_be_update_query castecne zajistuje detekci zda je mozne pouzit update dotaz - ovsem nemus fungovat dobre, pokud spojeni probiha na zaklade vyrazu jako substring
         $icon=1;
         $cols[0]{name}="#";  # update
         $cols[0]{type}="";
         $cols[0]{chosen}="9";  # zajisti, ze sloupec ikon bude v class="chosen9" - a nebude videt pri tisku
      }
      else {
         $icon=0;
      }
      #- zjisteni presnych jmen sloupcu v tabulkach ve tvaru database.table.col
      my @cols_sqlname=Maccess_query_string->query_string_cols_simple($query_string);
      #- zjisteni jmen sloupcu jejichz hodnoty jsou diky spojeni identicke s jinymi
      my ($cols_sqljoin)=Maccess_query_string->query_string_cols_join($query_string);
      #- zjisteni jmen sloupcu, ktere jsou primarnimi klici pro tabulky, ktere jsou v $query_string
      my @cols_prikey_all=Maccess_query_string->query_string_primary_keys($query_string);
      my @cols_prikey; # seznam primarnich klicu, ktere jsou zobrazovany
      my $col=$icon-1;
      for(my $col_sql=0;$col_sql<$num_fields;++$col_sql) {
         if($cols_maccess_type[$col_sql] ne "style" and $cols_maccess_type[$col_sql] ne "href" and $cols_maccess_type[$col_sql] ne "hidden_group") {
            ++$col;
            $cols[$col]{name}=$pkg->decode_maccess($$names[$col_sql]);
            $cols[$col]{type}=$pkg->field_type($$names_type[$col_sql]);
            if(defined($cols_sqlname[$col_sql])) {
               $cols[$col]{order_by}=$cols_sqlname[$col_sql];
               $cols[$col]{sqlname}=$cols_sqlname[$col_sql];
               if(grep(/^\Q$cols[$col]{sqlname}\E$/,@cols_prikey_all)) {
                  $cols[$col]{prikey}=1;
                  push(@cols_prikey,$cols[$col]{sqlname});
               }
               $cols[$col]{sqljoin}=$$cols_sqljoin{$cols[$col]{sqlname}};
               $cols[$col]{sqltype}=$pkg->field_sqltype_by_col_fullname($cols[$col]{sqlname});
               $cols[$col]{length}=$pkg->field_length($cols[$col]{sqlname});
            }
            else {
               # Pozn: vlastnost MySQL - pokud chci dle nejakeho vyrazu radit, je nutne ho pojmenovat - tedy to musim ucinit uz v navrhu dotazu, aby i pozdeji fungovalo razeni na kliknuti na nazvu sloupce
               $cols[$col]{order_by}=$$names[$col_sql];
            }
            if($cols_maccess_type[$col_sql] eq "xml_group") {
               $cols[$col]{xml_group}=1;
            }
         }
      }
      #- zjisteni, ktere tabulky v $query_string obsahuji vsechny primarni klice, a tedy jejichz polozky mohou byt aktualizovany
      my @tables_fullname_maybeupdate=$pkg->tables_maybeupdate(@cols_prikey);
      #- pridani pripadne znacky ke jmenu sloubce, ze muze byt aktualizovan
      for(my $col=0;$col<=$#cols;++$col) {
         if($cols[$col]{sqlname}=~/^([^\.]+)\.([^\.]+)\.([^\.]+)$/) {
            my $table_fullname="$1.$2";
            if(grep(/^\Q$table_fullname\E$/,@tables_fullname_maybeupdate)) {
               $cols[$col]{update}="1";
            }
         }
      }
      my $row=0;
      while(my $ref=$sth->fetchrow_arrayref) {
         my (%names_hash);
         my $col=$icon-1;
         for(my $col_sql=0;$col_sql<$num_fields;++$col_sql) {
            if($cols_maccess_type[$col_sql] ne "style" and $cols_maccess_type[$col_sql] ne "href" and $cols_maccess_type[$col_sql] ne "hidden_group") {
               ++$col;
               $data[$row][$col]{value}=$$ref[$col_sql];
            }
            elsif($cols_maccess_type[$col_sql] eq "style") {
               $data[$row][$col]{style}=$$ref[$col_sql];
            }
            elsif($cols_maccess_type[$col_sql] eq "href") {
               $data[$row][$col]{href}=$$ref[$col_sql];
            }
            $names_hash{$$names[$col_sql]}=$$ref[$col_sql];
         }
         if($icon) {
            my $href;
            # ungroup
            if($cols[0]{name} eq "+") {
               $href=$pkg->href_ungroup_by($id,\@cols_group,\%names_hash);
            }
            # update
            elsif($cols[0]{name} eq "#") {
               my $record=$row+$limit[0];
               $href=$pkg->href_update_record($id,$record);
            }
            $data[$row][0]{value}="<xhtml>";
            $data[$row][0]{value}.="<img";
            $data[$row][0]{value}.=" border=";
            $data[$row][0]{value}.="\"";
            $data[$row][0]{value}.=encode_entities("0");
            $data[$row][0]{value}.="\"";
            $data[$row][0]{value}.=" alt=";
            $data[$row][0]{value}.="\"";
            $data[$row][0]{value}.=encode_entities($cols[0]{name});
            $data[$row][0]{value}.="&#160;".encode_entities("(" . ($row+1) . ")");
            $data[$row][0]{value}.="\"";
            $data[$row][0]{value}.=" src=";
            $data[$row][0]{value}.="\"";
            $data[$row][0]{value}.=encode_entities(encode_uri("$maccess_lib_path/$cols[0]{name}.gif"));
            $data[$row][0]{value}.="\"";
            $data[$row][0]{value}.="/>";
            $data[$row][0]{value}.="</xhtml>";
            $data[$row][0]{href}=$href;
         }
         ++$row;
      }
      #-limit
      $limit=[$limit[0],$limit[1],$row];
   }
   #---- jde o krizovy dotaz
   else {
      my @cols_group;
      if($$param{link} eq "no") {
         $icon=0 ;
      }
      else {
         $icon=1;
         @cols_group=Maccess_query_string->query_string_cols_group($query_string);
      }
      my (@row_names,@col_names,%col_names_sql,%col_names_sql2,%col_names_type);
      # zjisteni nazvu budoucich sloupcu a nazvu radku a preskupeni hodnot
      my $num_fields_maccess_type_value=0;   # zjisteni skutecnych sloupcu ruznych hodnot
      for(my $col_sql=2;$col_sql<$num_fields;++$col_sql) {
         if($cols_maccess_type[$col_sql] eq "value") {
            ++$num_fields_maccess_type_value;
         }
      }
      my $row=0;
      while(my $ref=$sth->fetchrow_arrayref) {
         my (%names_hash);
         $names_hash{$$names[0]}=$$ref[0];
         $names_hash{$$names[1]}=$$ref[1];
         push(@row_names,$$ref[1]) if(!grep(/^\Q$$ref[1]\E$/, @row_names));
         my ($col_name);
         for(my $col_sql=2;$col_sql<$num_fields;++$col_sql) {
            if($cols_maccess_type[$col_sql] ne "style" and $cols_maccess_type[$col_sql] ne "href" and $cols_maccess_type[$col_sql] ne "hidden_group") {
               $col_name="<xhtml>";
               if($$ref[0]!~/^<xhtml>/) {
                  $col_name.=encode_entities($$ref[0]);
               }
               else {
                  $col_name.=$pkg->drop_xhtml($$ref[0]);
               }
               if($num_fields_maccess_type_value>1) {
                  $col_name.=" (";
                  if($pkg->decode_maccess($$names[$col_sql])!~/^<xhtml>/) {
                     $col_name.=encode_entities($pkg->decode_maccess($$names[$col_sql]));
                  }
                  else {
                     $col_name.=$pkg->drop_xhtml($pkg->decode_maccess($$names[$col_sql]));
                  }
                  $col_name.=")";
               }
               $col_name.="</xhtml>";
               push(@col_names,$col_name) if(!grep(/^\Q$col_name\E$/, @col_names));
               $col_names_sql{$col_name}=$pkg->decode_maccess($$names[$col_sql]);
               $col_names_sql2{$col_name}=$$ref[0];
               $col_names_type{$col_name}=$$names_type[$col_sql];
               $cross_ref{$$ref[1]}{$col_name}{value}=$$ref[$col_sql];
            }
            elsif($cols_maccess_type[$col_sql] eq "style") {
               $cross_ref{$$ref[1]}{$col_name}{style}=$$ref[$col_sql];
            }
            elsif($cols_maccess_type[$col_sql] eq "href") {
               $cross_ref{$$ref[1]}{$col_name}{href}=$$ref[$col_sql];
            }
            $names_hash{$$names[$col_sql]}=$$ref[$col_sql];
            # ungroup
            if(!$cross_ref{$$ref[1]}{$col_name}{href} and $icon) {
               my $href=$pkg->href_ungroup_by($id,\@cols_group,\%names_hash);
               $cross_ref{$$ref[1]}{$col_name}{href}=$href;
            }
         }
         ++$row;
      }
      @col_names=@{$pkg->sort_array(\@col_names,"","asc")}; # vzdy radim alfanumericky a ne dle typu $pkg->field_type($$names_type[0]); je to proto, ze do nazvy doplnuji dalsi textove sekvence a prevod na cisla a razeni dle toho nemusi byt ok
      @row_names=@{$pkg->sort_array(\@row_names,$pkg->field_type($$names_type[1]),"asc")};
      $cols[0]{name}=$pkg->decode_maccess($$names[1]);
      $cols[0]{type}=$pkg->field_type($$names_type[1]);
      $cols[0]{order_by}=$cols[0]{name};
      my $col=1;
      foreach my $col_name (@col_names) {
         $cols[$col]{name}=$col_name;
         $cols[$col]{name_sql}=$col_names_sql{$col_name};
         $cols[$col]{type}=$pkg->field_type($col_names_type{$col_name});
         $cols[$col]{order_by}=$cols[$col]{name};
         ++$col;
      }
      # pridani nazvu budoucich sloupcu Suma
      if($$param{query_type}!~/^graph/) { # nepripojuj v pravo sumu pokud jde o graf
         my $korekce=0;
         my $col=0;
         foreach my $col_name (@col_names) {
            ++$col;
            my $modulo=(($col-1) % $num_fields_maccess_type_value);
            if($modulo==0) { # dalsi sada sloupcu
               $korekce=0;
            }
            if($cols[$col]{type} eq "number") {
               my $names_sum="<xhtml>";
               $names_sum.="&#931;";
               if($num_fields_maccess_type_value>1) {
                  $names_sum.=" (";
                  $names_sum.=$cols[$col]{name_sql};
                  $names_sum.=")";
               }
               $names_sum.="</xhtml>";
               $cols[1+$#col_names+1+$modulo+$korekce]{name}=$names_sum;
               $cols[1+$#col_names+1+$modulo+$korekce]{type}=$cols[$col]{type};
               $cols[1+$#col_names+1+$modulo+$korekce]{order_by}=$cols[1+$#col_names+1+$modulo+$korekce]{name};
            }
            else {
               --$korekce;
            }
         }
      }
      # zaznamenani preskupenych dat, pocitani a zaznamenani sumy
      my $row=0;
      foreach my $row_name (@row_names) {
         my $korekce=0;
         my $col=0;
         $data[$row][$col]{value}=$row_name;
         my @col_sum=();
         foreach my $col_name (@col_names) {
            ++$col;
            $data[$row][$col]{value}=$cross_ref{$row_name}{$col_name}{value};
            $data[$row][$col]{style}=$cross_ref{$row_name}{$col_name}{style};
            $data[$row][$col]{href}=$cross_ref{$row_name}{$col_name}{href};
            my $modulo=(($col-1) % $num_fields_maccess_type_value);
            if($modulo==0) { # dalsi sada sloupcu
               $korekce=0;
            }
            if($cols[$col]{type} eq "number") {
              $data[$row][1+$#col_names+1+$modulo+$korekce]{value}+=$cross_ref{$row_name}{$col_name}{value};
            }
            else {
               --$korekce;
            }
         }
         ++$row;
      }
      # razeni dat
      if(defined($$param{query_order_by_name})) {
         # ziskani cisla slopupce dle ktereho chci radit
         my $col_order_by;
         for(my $col=0;$col<=$#cols;++$col) {
            if($cols[$col]{name} eq $$param{query_order_by_name}) {
               $col_order_by=$col;
               last;
            }
         }
         # samotne serazeni pole @data obsahujici vsechny zaznamy
         if(defined($col_order_by)) {
            my $data_value;
            for(my $row=0;$row<=$#data;++$row) {
               $$data_value[$row]=$data[$row][$col_order_by]{value};
            }
            my @ival=$pkg->sort_array_ival($data_value,$cols[$col_order_by]{type},$$param{query_order_by_sort});
            my @data_sort;
            for(my $row=0;$row<=$#data;++$row) {
               $data_sort[$row]=$data[$ival[$row]];
            }
            @data=@data_sort;
         }
      }
   }
   $sth->finish;
   # doplneni udaju do param pro pohodlne vyvolani uprav na specialni nastaveni tabulky
   delete($$param{cols_maccess_spec_name});
   for(my $col=0;$col<=$#cols;++$col) {
      push(@{$$param{cols_maccess_spec_name}},{ name => $cols[$col]{name}, type => $cols[$col]{type} });
   }
   # zaznamenani udaju v param do session - znovu, jelikoz jsem doplnoval $$param{cols_maccess_spec_name}
   $pkg->sessionwrite($id,$param);
   return (\@cols,\@data,$id,$limit);
}

#= ------------- Odeslani pozadavku SQL serveru a zobrazeni vysledku --------- =#
sub query_print {
   my ($pkg_p, $param)=@_;
   my $query_string=$$param{query_string};
   my $query=MyCGI->cgisession;
   # Funkce prevalcuje nektere hodnoty v param, pokud prisli primo GET ci POST metodou v pozadavku
   $param=$pkg->update_param_by_GET_or_POST($param,$query);
   # Spusteni prislusne fce dle hodnoty $$param{query_type}
   if($$param{query_type}=~/^graph/) {
      $pkg->query_print_graph($param);
   }
   elsif($$param{query_type}=~/^map/) {
      $pkg->query_print_map($param);
   }
   elsif($$param{query_type} eq "update") {
      my ($query_string_no_limit, @limit)=Maccess_query_string->query_string_no_limit($query_string);
      if($limit[1]!=1) {
         my $query_hash=Maccess_query_string->query_string_to_query_hash($query_string);
         $$query_hash{LIMIT}="1";
         $query_string=Maccess_query_string->query_hash_to_query_string($query_hash);
      }
      $pkg->query_print_xml_table($param);
   }
   elsif($$param{query_type} eq "update_add") {
      my $query_hash=Maccess_query_string->query_string_to_query_hash($query_string);
      undef($$query_hash{WHERE});
      undef($$query_hash{"GROUP BY"});
      undef($$query_hash{HAVING});
      undef($$query_hash{'ORDER BY'});
      $$query_hash{LIMIT}="1";
      $query_string=Maccess_query_string->query_hash_to_query_string($query_hash);
      $pkg->query_print_xml_table($param);
   }
   else {
      $pkg->query_print_xml_table($param);
   }
}

#= ------------- Zobrazeni ve forme GRAF ------------------------------------- =#
sub query_print_graph {
   my ($pkg_p, $param)=@_;
   # zaznamenani udaju v param do session
   my $id=$pkg->sessionwrite(undef,$param);
   my $query=MyCGI->cgisession;
   my $idcs=$query->param('_idcs');
   my $outputdev=$pkg->outputdev($query);
   my ($docO,$htmlO,$bodyO)=$pkg->creahtml(
      $outputdev,
      $idcs,
      {
         h1 => $$param{query_name},
         query_id => $$param{query_id},
         folder => $$param{folder}
      }
   );
   $pkg->outputvar($bodyO,$$param{variables},$outputdev);
   # samotny obrazek grafu
   my $img_src=My->okruh;
   $img_src.="?id=";
   $img_src.=encode_uri($id);
   $img_src.="&graph_gif=";
   $img_src.=encode_uri("1");
   my $pO=$bodyO->aT("p");
   $pO->aT(
      "img",
      {
         src => $img_src
      }
   );
   # spec nastaveni - ukladani pro graf
   if(
      length($$param{query_id}) and
      Maccess->create_user_query($$param{query_id}) eq Maccess->get_remote_user and
      !defined($pkg->query_id_system)
   ) {
      my $grid_field_spec=$pkg->query_file_spec_to_hash($$param{query_id},$$param{query_type},"","");
      my $divO=$bodyO->aT(
         "div",
         {
            class => "button"
         }
      );
      $divO->aT("b", {}, translate("Spec. settings:"));
      $divO->aT("br");
      my $formO=$divO->aT(
         "form",
         {
            method => "post",
            action => $maccess_bin_path."/query_spec_graph.phtml"
         }
      );
      my $divOO=$formO->aT("div");
      $divOO->aT(
         "input",
         {
            name => "id",
            type => "hidden",
            value => $id
         }
      );
      if($$param{query_type} ne "graph_pie") {
         $divOO->aT(undef,{},translate("y min:"));
         $divOO->aT(undef,{}," ");
         $divOO->aT(
            "input",
            {
               name => "min_val",
               type => "text",
               size => "6",
               value => $$grid_field_spec{min_val}
            }
         );
         $divOO->aT(undef,{},"\x{a0}");
         $divOO->aT(undef,{},translate("y max:"));
         $divOO->aT(undef,{}," ");
         $divOO->aT(
            "input",
            {
               name => "max_val",
               type => "text",
               size => "6",
               value => $$grid_field_spec{max_val}
            }
         );
         $divOO->aT(undef,{},"\x{a0}");
         $divOO->aT(undef,{},translate("divide y:"));
         $divOO->aT(undef,{}," ");
         $divOO->aT(
            "input",
            {
               name => "y_ticks",
               type => "text",
               size => "6",
               value => $$grid_field_spec{y_ticks}
            }
         );
         $divOO->aT(undef,{},"\x{a0}");
         $divOO->aT(
            "input",
            {
               name => "save",
               type => "submit",
               value => translate("Save")
            }
         );
      }
   }
   Output->outputprint($docO,$outputdev);
   return 1;
}

#= ------------- Zobrazeni ve forme GRAF - print gifu ------------------------ =#
sub query_print_graph_gif {
   my ($pkg_p,$param)=@_;
   my ($cols,$data)=$pkg->query_to_array($param);
   my $query_name=$$param{query_name};
   my $num_rows=@$data;
   my $num_fields=@$cols;
   my $graph_x_label=$pkg->graph_popis($$cols[0]{name});
   if(Maccess_locale->charset eq "windows-1250") { ## typicky ceska zalezitost
      $query_name=$pkg->recode_charset($query_name,"windows-1250","iso-8859-2");
      $graph_x_label=$pkg->recode_charset($graph_x_label,"windows-1250","iso-8859-2");
   }
   my $legend_length=1;
   my @graph_legend;
   for(my $i=1;$i<$num_fields;$i++) {
      $graph_legend[$i-1]=$pkg->graph_popis($$cols[$i]{name});
      if(Maccess_locale->charset eq "windows-1250") { ## typicky ceska zalezitost
         $graph_legend[$i-1]=$pkg->recode_charset($graph_legend[$i-1],"windows-1250","iso-8859-2");
      }
      $legend_length=length($graph_legend[$i-1]) if($legend_length<length($graph_legend[$i-1]));
   }
   my $labels_length=1;
   my $row=0;
   my @graph_data;
   while($row<$num_rows) {
      $graph_data[0] [$row] = $$data[$row][0]{value};
      if(Maccess_locale->charset eq "windows-1250") { ## typicky ceska zalezitost
         $graph_data[0] [$row] = $pkg->recode_charset($graph_data[0] [$row],"windows-1250","iso-8859-2");
      }
      $labels_length=length($$data[$row][0]{value}) if($labels_length<length($$data[$row][0]{value}));
      for(my $i=1;$i<$num_fields;++$i) {
         $graph_data[$i] [$row] = $$data[$row][$i]{value};
      }
      ++$row;
   }
   my $labels_numbers=$num_rows;
   # do grafu se na osu x vleze asi 90 pismenek, chtel bych maximalne asi jen (90/label_lenght) labelu (popisek osy x)
   my $lebels_max_letters=80; # je to fakt viladeno jak na 2pismenovejch popiskach tak i na dlouhejch
   $lebels_max_letters-=$legend_length;  # v pravo zobrazujes legendy y-novych hodnot a ty mohou mit promenou delku  # pokud jde o hodnoty na ose y predpokladam, ze nejsou prilis dlouhe ale nejak o tom neuvazuji
   # Za urcitych podminek tiskni popisky osy x vertikalne
   my $graph_x_labels_style="normal";
   my $graph_x_labels_skip;
   if(($labels_length>=5) && ($labels_length<=15) && (($labels_length*$labels_numbers)>$lebels_max_letters)) {
      $graph_x_labels_style="vertical";
      $graph_x_labels_skip=int( 2.5*$labels_numbers / $lebels_max_letters ) + 1;  # rekneme, ze pismenko je 2.5 krat vyssi nez sirsi
   }
   else {
      $graph_x_labels_skip=int( $labels_length*$labels_numbers / $lebels_max_letters ) + 1;
   }
   my $graph_grey_background=[230,230,230];
   my ($gifx,$gify)=(650,400);
   # nacteni spec nastaveni
   my $grid_field_spec=$pkg->query_file_spec_to_hash($$param{query_id},$$param{query_type},"","");
   my $my_graph;
   # individualni nastaveni pro kazdy graf
   if($$param{query_type} eq "graph_bars") {
      require Chart::Bars;
      $my_graph=Chart::Bars->new($gifx,$gify);
      $my_graph->set("x_label"=>$graph_x_label);
      $my_graph->set("grid_lines"=>"true");
      $my_graph->set("x_ticks"=> $graph_x_labels_style);
      $my_graph->set("skip_x_ticks"=> $graph_x_labels_skip);
      $my_graph->set("min_val"=>$$grid_field_spec{min_val}) if(length($$grid_field_spec{min_val}));
      $my_graph->set("max_val"=>$$grid_field_spec{max_val}) if(length($$grid_field_spec{max_val}));
      $my_graph->set("y_ticks"=>($$grid_field_spec{y_ticks}+1)) if(length($$grid_field_spec{y_ticks}));
   }
   elsif($$param{query_type} eq "graph_bars_sum") {
      require Chart::StackedBars;
      $my_graph=Chart::StackedBars->new($gifx,$gify);
      $my_graph->set("x_label"=>$graph_x_label);
      $my_graph->set("grid_lines"=>"true");
      $my_graph->set("x_ticks"=> $graph_x_labels_style);
      $my_graph->set("skip_x_ticks"=> $graph_x_labels_skip);
      $my_graph->set("min_val"=>$$grid_field_spec{min_val}) if(length($$grid_field_spec{min_val}));
      $my_graph->set("max_val"=>$$grid_field_spec{max_val}) if(length($$grid_field_spec{max_val}));
      $my_graph->set("y_ticks"=>($$grid_field_spec{y_ticks}+1)) if(length($$grid_field_spec{y_ticks}));
   }
   elsif($$param{query_type} eq "graph_pie") {
      require Chart::Pie;
      $my_graph=Chart::Pie->new($gifx,$gify);
   }
   elsif($$param{query_type} eq "graph_area") {
      require Chart::Mountain;
      $my_graph=Chart::Mountain->new($gifx,$gify);
      $my_graph->set("x_label"=>$graph_x_label);
      $my_graph->set("grid_lines"=>"true");
      $my_graph->set("x_ticks"=> $graph_x_labels_style);
      $my_graph->set("skip_x_ticks"=> $graph_x_labels_skip);
      $my_graph->set("min_val"=>$$grid_field_spec{min_val}) if(length($$grid_field_spec{min_val}));
      $my_graph->set("max_val"=>$$grid_field_spec{max_val}) if(length($$grid_field_spec{max_val}));
      $my_graph->set("y_ticks"=>($$grid_field_spec{y_ticks}+1)) if(length($$grid_field_spec{y_ticks}));
   }
   elsif($$param{query_type} eq "graph_lines") {
      require Chart::Lines;
      $my_graph=Chart::Lines->new($gifx,$gify);
      $my_graph->set("x_label"=>$graph_x_label);
      $my_graph->set("grid_lines"=>"true");
      $my_graph->set("brush_size"=> 3);
      $my_graph->set("x_ticks"=> $graph_x_labels_style);
      $my_graph->set("skip_x_ticks"=> $graph_x_labels_skip);
      $my_graph->set("min_val"=>$$grid_field_spec{min_val}) if(length($$grid_field_spec{min_val}));
      $my_graph->set("max_val"=>$$grid_field_spec{max_val}) if(length($$grid_field_spec{max_val}));
      $my_graph->set("y_ticks"=>($$grid_field_spec{y_ticks}+1)) if(length($$grid_field_spec{y_ticks}));
   }
   elsif($$param{query_type} eq "graph_linespoints") {
      require Chart::LinesPoints;
      $my_graph=Chart::LinesPoints->new($gifx,$gify);
      $my_graph->set("x_label"=>$graph_x_label);
      $my_graph->set("grid_lines"=>"true");
      $my_graph->set("brush_size"=> 3);
      $my_graph->set("x_ticks"=> $graph_x_labels_style);
      $my_graph->set("skip_x_ticks"=> $graph_x_labels_skip);
      $my_graph->set("min_val"=>$$grid_field_spec{min_val}) if(length($$grid_field_spec{min_val}));
      $my_graph->set("max_val"=>$$grid_field_spec{max_val}) if(length($$grid_field_spec{max_val}));
      $my_graph->set("y_ticks"=>($$grid_field_spec{y_ticks}+1)) if(length($$grid_field_spec{y_ticks}));
   }
   # spolecna nastaveni pro vsechny grafy
   $my_graph->set("transparent"=>"true");
   $my_graph->set("title"=>$query_name);
   $my_graph->set("legend_labels" => \@graph_legend);
   my $dset = 0;
   $my_graph->set(
      "colors" =>
      {
         "grey_background" => $graph_grey_background,
         "x_grid_lines" => $graph_grey_background, # sladeno s barvou pozadi, aby nebylo videt
         (
            map { 'dataset'.$dset++ => $_ }
               qw (
                  OrangeRed1 RoyalBlue1 green4 DarkGoldenrod3 salmon1
                  HotPink3 DarkOrchid1 SteelBlue3 orange2 LightPink4
                  PaleGreen3
               )
         )
      }
   );
   # vystup grafu
   $my_graph->cgi_png(\@graph_data);
}

#= --------------- Zobrazeni ve forme mapy ----------------------------------- =#
## chce to jeste dodelat, aby pri prejizdeni nad statem vyskocila bublina s hodnotou
sub query_print_map {
   my ($pkg_p,$param)=@_;
   require GD;
   my ($cols,$data)=$pkg->query_to_array($param);
   my $num_rows=@$data;
   my $map_table=$$param{query_type};
   my $im = new GD::Image(555,674); ## Rozmery mapy - chce zaznamenat nekde mimo program (muzu mit ruznorode mapy)
   my $blue = $im->colorAllocate(150, 189, 255);
   my $black = $im->colorAllocate(0, 0, 0);
   my $gray = $im->colorAllocate(200, 200, 200);
   # Definovani barev
   my @color;
   my $num_colors=200;
   for(my $i=0; $i<=$num_colors; ++$i) {
      $color[$i]=$im->colorAllocate(250-200/$num_colors*$i,int(250-($i/3)),250-200/$num_colors*$i);
   }
   # Zjisteni maximalni a minimalni hodnoty
   my ($max_val, $min_val);
   my $row=0;
   my %object;
   while($row<$num_rows) {
      $object{$$data[$row][0]{value}}=$$data[$row][1]{value};
      if($$data[$row][1]{value}>$max_val) {
         $max_val=$$data[$row][1]{value};
      }
      if($$data[$row][1]{value}<$min_val) {
         $min_val=$$data[$row][1]{value};
      }
      ++$row;
   }
   my $sth_Maps=$dbh_sys->prepare(q{
      select
         } . $map_table . q{.code,
         } . $map_table . q{.def
      from
         } . $map_table . q{
   });
   # Vytvoreni mapy
   my $rv_Maps=$sth_Maps->execute;
   while(my $ref_Maps=$sth_Maps->fetchrow_arrayref) {
      my $color=$gray;
      my $color_index=int( ($object{$$ref_Maps[0]}-$min_val) / (($max_val-$min_val)/$num_colors) );
      $color=$color[$color_index] if($object{$$ref_Maps[0]});
      $pkg->draw_country($$ref_Maps[1],$im,$color,$black);
   }
   my $rc_Maps=$sth_Maps->finish;
   # Vytvoreni popisek mapy
   my $rv_Maps=$sth_Maps->execute;
   while(my $ref_Maps=$sth_Maps->fetchrow_arrayref) {
      $pkg->draw_code($$ref_Maps[1],$im,$black,$$ref_Maps[0]);
   }
   my $rc_Maps=$sth_Maps->finish;
   print "Content-Type: image/png\n\n";
   binmode STDOUT;
   print $im->png;
}

# pomocna funkce pro zobrazeni ve forme mapy
sub draw_country {
   my ($pkg_p, $def, $myim, $color, $color_border)=@_;
   foreach my $radek (split(/\n/,$def)) {
      if($radek=~/^.+,.+,/) {
         my $poly=new GD::Polygon;
         my @border=split(/,/,$radek);
         while(@border) {
            my $x=shift @border;
            my $y=shift @border;
            $poly->addPt($x,$y);
         }
         $myim->filledPolygon($poly,$color);
         $myim->polygon($poly,$color_border);
      }
   }
}

# pomocna funkce pro zobrazeni ve forme mapy
sub draw_code {
   my ($pkg_p, $def, $myim, $color, $code)=@_;
   $def=~s/^(.+)$/$1/;
   my ($x,$y)=split(/,/,$def);
   $myim->string(GD::Font->Large,$x-4,$y-5,$code,$color);
}

#= ----------------- Zobrazeni ve forme XML ---------------------------------- =#
sub query_print_xml_table {
   my ($pkg_p, $param)=@_;
   #- spusteni dotazu
   my ($cols,$data,$id,$limit)=$pkg->query_to_array($param);
   #- vytvoreny xml tabulky
   my ($docM,$tableM,$colsM,$rowsM)=MyXML->creatable;
   #if($$param{query_type} eq "update" or $$param{query_type} eq "update_add") {
   #   my $name_stylesheet= $maccess_lib_path . "/template_" . $$param{query_type} . ".xsl";
   #   my $pi=$docM->createProcessingInstruction("xml-stylesheet", 'type="text/xsl" href="'.$name_stylesheet.'"');
   #   $docM->insertBefore($pi,$tableM);
   #}
   $pkg->query_print_xml_body_cols($colsM,$param,$cols,$data,$id,$limit);
   $pkg->query_print_xml_body_rows($rowsM,$param,$cols,$data,$id,$limit);
   if($$param{query_type} eq "update" or $$param{query_type} eq "update_add") {
      my $query=MyCGI->cgisession;
      my $idcs=$query->param('_idcs');
      my $outputdev=$pkg->outputdev($query);
      delete($$outputdev{bezpre});
      my ($docO,$htmlO,$bodyO)=$pkg->creahtml(
         $outputdev,
         $idcs,
         {
            h1 => $$param{query_name},
            query_id => $$param{query_id},
            folder => $$param{folder}
         }
      );
      $pkg->outputvar($bodyO,$$param{variables},$outputdev);
      $pkg->query_print_update(
         $bodyO,
         $tableM,
         $outputdev,
         $param,
         $id,
         $query->param('id') # originalni id
      );
      Output->outputprint($docO,$outputdev);
   }
   elsif($$param{query_type} eq "xml_raw") {
      print "Content-type: text/xml\n\n";
      print $docM->toString;
   }
   else {
      #- vytvoreni vystupniho kodu vystupu
      my $query=MyCGI->cgisession;
      my $idcs=$query->param('_idcs');
      my $outputdev=$pkg->outputdev($query);
      delete($$outputdev{bezpre});
      my ($docO,$htmlO,$bodyO);
      ($docO,$htmlO,$bodyO,$$outputdev{prvniradek})=$pkg->creahtml(
         $outputdev,
         $idcs,
         {
            h1 => $$param{query_name},
            query_id => $$param{query_id},
            folder => $$param{folder}
         }
      );
      if(length($$param{query_id}) and !defined($pkg->query_id_system)) {
         # zmena standadtni maccessovske hlavicky
         if(my ($headO)=$htmlO->findnodes('head')) {
            bless($headO,"MyXML");
            my $href=$pkg->href_spec_xml_table($id);
            my $sriptO=$headO->aT(
               "script",
               {
                  type => "text/javascript"
               },
               q/
                  myMenuTable[0]=new Array(5);
                  myMenuTable[0][0]="/ . translate("set filter") . q/";
                  myMenuTable[0][1]="javascript:tableFilter(0)";
                  myMenuTable[0][2]="chosen0";
                  myMenuTable[1]=new Array(5);
                  myMenuTable[1][0]="/ . translate("set filter") . q/";
                  myMenuTable[1][1]="javascript:tableFilter(1)";
                  myMenuTable[1][2]="chosen1";
                  myMenuTable[2]=new Array(5);
                  myMenuTable[2][0]="/ . translate("set filter") . q/";
                  myMenuTable[2][1]="javascript:tableFilter(2)";
                  myMenuTable[2][2]="chosen2";
                  myMenuTable[3]=new Array(5);
                  myMenuTable[3][0]="/ . translate("disable filter") . q/";
                  myMenuTable[3][1]="javascript:tableFilter(-1)";
                  myMenuTable[4]=new Array(5);
                  myMenuTable[4][0]="/ . translate("unmark all") . q/";
                  myMenuTable[4][1]="javascript:tableFilterMarkAll(0)";
               /
            );
            if(
               length($$param{query_id}) and
               Maccess->create_user_query($$param{query_id}) eq Maccess->get_remote_user and
               !defined($pkg->query_id_system)
            ) {
               $sriptO->aT(
                  undef,
                  {},
                  q/
                     myMenuTable[5]=new Array(5);
                     myMenuTable[5][0]="/ . translate("spec. settings") . q/";
                     myMenuTable[5][1]="/.$href.q/";
                  /
               );
            }
         }
      }
      if(defined($limit)) {
         push(@{$$param{variables}}, [translate("Limit"), ($$limit[0]+1)."-".($$limit[0]+$$limit[2])] );
      }
      $$outputdev{prvniradek}=$pkg->outputvar($bodyO,$$param{variables},$outputdev);
      $$outputdev{prvniradek}=Output->output(
         $bodyO,
         $tableM,
         $outputdev
      );
      if(defined($limit) and $$limit[2]==$$limit[1]) {
         my $href_limit_records=$pkg->href_limit_records($id,[$$limit[0],$$limit[1]]);
         my $pO=$bodyO->aT("p");
         $pO->aT(
            "a",
            {
               href => $href_limit_records
            },
            translate("next :0 rows",$$limit[1])
         );
      }
      Output->outputprint($docO,$outputdev);
   }
   return 1;
}

#---- tisk XML tela - definice sloupcu
sub query_print_xml_body_cols {
   my ($pkg_p, $colsM,$param,$cols,$data,$id,$limit)=@_;
   my $num_fields=@$cols;
   #- XML cast "cols"
   my $spec_type="xml_table";
   for(my $col=0;$col<$num_fields;$col++) {
      my $grid_field_spec=$pkg->query_file_spec_to_hash($$param{query_id},$spec_type,$$cols[$col]{name},$col);
      if(!defined($$cols[$col]{chosen})) {
         if(defined($$grid_field_spec{chosen})) {
            $$cols[$col]{chosen}= $$grid_field_spec{chosen};
         }
         else {
            $$cols[$col]{chosen}= "0";
         }
      }
      $$cols[$col]{format}=$$grid_field_spec{format};
      $$cols[$col]{maccess_formatfce}=$pkg->query_file_spec_to_hash_ffce($$param{query_id},$spec_type,$$cols[$col]{name},$col,$cols);
      if($$cols[$col]{type} eq "number") {
         if(!defined($$grid_field_spec{summary})) {
            $$cols[$col]{summary}="sum";
         }
         elsif($$grid_field_spec{summary} eq "none") {
            $$cols[$col]{summary}=undef;
         }
         else {
            $$cols[$col]{summary}=$$grid_field_spec{summary};
         }
      }
      $$cols[$col]{hidden}=$$grid_field_spec{hidden};
      if(!$$cols[$col]{xml_group}) {
         my $colM=$colsM->aT(
            "col",
            {
               name => $$cols[$col]{name},
               type => $$cols[$col]{type},
               format => $$cols[$col]{format},
               summary => $$cols[$col]{summary},
               chosen => $$cols[$col]{chosen},
               hidden => $$cols[$col]{hidden}
            }
         );
         if(defined($$cols[$col]{order_by})) {
            my $href=$pkg->href_order_by($id,$$cols[$col]{order_by});
            $colM->sA("style","cursor: pointer;");
            $colM->sA("onclick","document.location.replace('" . $href . "'); return false;");
         }
         if($$cols[$col]{name}!~/^<xhtml>/) {
            $colM->aT(undef,{},$$cols[$col]{name});
         }
         else {
            $colM->aNchilds_fromStringHTML($$cols[$col]{name});
         }
         $colM->sA("sqltype",$$cols[$col]{sqltype}) if($$cols[$col]{sqltype});
         $colM->sA("sqlname",$$cols[$col]{sqlname}) if($$cols[$col]{sqlname});
         $colM->sA("sqljoin",$$cols[$col]{sqljoin}) if($$cols[$col]{sqljoin});
         $colM->sA("prikey",$$cols[$col]{prikey}) if($$cols[$col]{prikey});
         $colM->sA("update",$$cols[$col]{update}) if($$cols[$col]{update});
         $colM->sA("length",$$cols[$col]{length}) if($$cols[$col]{length});
      }
   }
}

#---- tisk XML tela - hodnoty
sub query_print_xml_body_rows {
   my ($pkg_p, $rowsM,$param,$cols,$data,$id,$limit)=@_;
   $pkg->print_rows($rowsM,$data,[],$cols,0,0)
}

#- tisk skupin radku v XML vystupu
sub print_rows {
   my ($pkg_p, $rowsM,$data,$sum,$cols,$number,$group_depth)=@_;
   my $num_rows=$#{$data}+1;
   my ($xml_group_values_row_all)=$pkg->xml_group_values($data,$cols,$number,undef);
   my $num_depths=$#{$xml_group_values_row_all}+1;
   $$sum[$group_depth]=undef;
   my $rowsMM;
   if($group_depth==0) {
      $rowsMM=$rowsM;
   }
   else {
      $rowsMM=$rowsM->aT("rows");
      # tisk nadpisu
      my ($xml_group_values,$col_group)=$pkg->xml_group_values($data,$cols,$number,$group_depth);
      $pkg->aTm(
         $rowsMM,
         "rows_heading",
         $col_group,       # cislo sloupce jehoz hodnotu chci tisknout
         $$data[$number],  # hodnoty celeho radku
         $cols             # infomrmace o vsech sloupcich
      );
   }
   if($group_depth==$num_depths) {
      my ($xml_group_values,$col_group)=$pkg->xml_group_values($data,$cols,$number,$group_depth+1);
      my $xml_group_values_row=$xml_group_values;
      while($pkg->is_array_equivalent($xml_group_values,$xml_group_values_row) and $number<$num_rows) {
         # tisk radku
         $number=$pkg->print_row($rowsMM,$data,$sum,$cols,$number,$group_depth);
         ($xml_group_values_row)=$pkg->xml_group_values($data,$cols,$number,$group_depth+1);
      }
   }
   else {
      my ($xml_group_values,$col_group)=$pkg->xml_group_values($data,$cols,$number,$group_depth);
      my $xml_group_values_row=$xml_group_values;
      while($pkg->is_array_equivalent($xml_group_values,$xml_group_values_row) and $number<$num_rows) {
         # tisk dalsi skupiny
         $number=$pkg->print_rows($rowsMM,$data,$sum,$cols,$number,$group_depth+1);
         ($xml_group_values_row)=$pkg->xml_group_values($data,$cols,$number,$group_depth);
         # navyseni sumy za skupinu
         my $num_fields=@$cols;
         for(my $col=0;$col<$num_fields;++$col) {
            if(!$$cols[$col]{xml_group}) {
               if($$cols[$col]{type} eq "number" and $$cols[$col]{summary} eq "sum") {
                  $$sum[$group_depth][$col]{value}+=$$sum[$group_depth+1][$col]{value};
               }
            }
         }
      }
   }
   # tisk radku se souhrnem - zvolene reseni vychazi se scitani hodnot vracene SQL servrem, pozdeji by bylo lepsi implementovat mnohem narocnejsi reseni, kdy budou i souhrny pocitany pres samostatne SQL dotazy (vyhoda: opravdovy celkovy souhrn i pri limitu radku, moznost aplikace dalsich souhrnych fci poskytovanych SQL servrem jako avg,...); doporucoval bych asi tohle delat po pripadnem predelani fce query_to_array ido XML podoby a neuzivani poli
   if(defined($$sum[$group_depth])) {
      my $rows_summaryM=$rowsMM->aT(
         "rows_summary",
         {
            chosen => "0"
         }
      );
      my $num_fields=@$cols;
      for(my $col=0;$col<$num_fields;++$col) {
         if(!$$cols[$col]{xml_group}) {
            # tisk znacek se sohrnou hodnotou
            $pkg->aTm(
               $rows_summaryM,
               "col",
               $col,                # cislo sloupce jehoz hodnotu chci tisknout
               $$sum[$group_depth], # hodnoty celeho radku
               $cols                # infomrmace o vsech sloupcich
            );
         }
      }
   }
   return $number;
}

#- tisk radku v XML vystupu
sub print_row {
   my ($pkg_p, $rowsM,$data,$sum,$cols,$number,$group_depth)=@_;
   my $rowM=$rowsM->aT(
      "row",
      {
         number => $number,
         chosen => "0"
      }
   );
   my $num_fields=@$cols;
   for(my $col=0;$col<$num_fields;++$col) {
      if(!$$cols[$col]{xml_group}) {
         # tisk znacek s hodnotou
         $pkg->aTm(
            $rowM,
            "col",
            $col,            # cislo sloupce jehoz hodnotu chci tisknout
            $$data[$number], # hodnoty celeho radku
            $cols            # infomrmace o vsech sloupcich
         );
         # pocitani souhrnych hodnot
         if($$cols[$col]{type} eq "number" and $$cols[$col]{summary} eq "sum") {
            $$sum[$group_depth][$col]{value}+=$$data[$number][$col]{value};
         }
      }
   }
   ++$number;
   return $number;
}

#- funkce vraci seznam hodnot sloupcu, ktere jsou xml_group
sub xml_group_values {
   my ($pkg_p,$data,$cols,$number,$group_depth_max)=@_;
   my @xml_group_values;
   my $col;
   if(!defined($group_depth_max) or $group_depth_max>0) {
      my $group_depth=0;
      my $num_fields=@$cols;
      for($col=0;$col<$num_fields;++$col) {
         if($$cols[$col]{xml_group}) {
            ++$group_depth;
            push(@xml_group_values,$$data[$number][$col]{value});
            last if(defined($group_depth_max) and $group_depth==$group_depth_max);
         }
      }
   }
   return (\@xml_group_values,$col);
}

#- pomocna funkce zjistujici, zda jsou dve pole ekvivalentni
sub is_array_equivalent {
   my ($pkg_p,$a1,$a2)=@_;
   return 0 if($#{$a1}!=$#{$a2});   # pole maji ruznou delku
   for(my $i=0;$i<=$#{$a1};++$i) {  # poravnavani prvku po prvku
      return 0 if($$a1[$i] ne $$a2[$i]);
   }
   return 1;
}

#- tisk xml znacek a hodnot
sub aTm {
   my ($pkg_p, $tM,$tagname,$col,$datarow,$cols)=@_;
   # nadefinovani casto pouzivanych hodnot ve fce
   my $value=$$datarow[$col]{value};
   my $style=$$datarow[$col]{style};
   my $href=$$datarow[$col]{href};
   my $type=$$cols[$col]{type};
   my $maccess_formatfce=$$cols[$col]{maccess_formatfce};
   # konec nadefinovani
   my $valueA;
   #- uprava 1
   if($type eq "number") {
      if(!defined($value) or $value eq '') {
         $type="";
      }
   }
   elsif($type eq "date") {
      my ($year,$month,$day)=split(/-/,$value);
      if(!Date::Calc::check_date($year,$month,$day)) {
         $type="";
         if(defined($value) and $value ne "") {
            $style="text-decoration: line-through; ".$style;
         }
      }
   }
   elsif($type eq "time") {
      if(!($value=~/^(\d\d):[0-5]\d:[0-5]\d$/ and $1<=24)) {
         $type="";
         if(defined($value) and $value ne "") {
            $style="text-decoration: line-through; ".$style;
         }
      }
   }
   elsif($type eq "dateTime") {
      $valueA=$value if(!defined($valueA));
      if($value=~/^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})$/) {   # jde o timestamp ve tvaru YYYYMMDDHHMMSS
         $value=$1."-".$2."-".$3." ".$4.":".$5.":".$6;
      }
      my ($date,$time)=split(/ /,$value);
      my ($year,$month,$day)=split(/-/,$date);
      if(!Date::Calc::check_date($year,$month,$day) or !($time=~/^(\d\d):[0-5]\d:[0-5]\d$/ and $1<=24)) {
         $type="";
         if(defined($value) and $value ne "") {
               $style="text-decoration: line-through; ".$style;
         }
      }
      else {
         $value=$date."T".$time;
      }
   }
   #- tvorba znacky
   my $tMM=$tM->aT($tagname);
   if(defined($$maccess_formatfce[0]) and defined($value)) {
      $valueA=$value if(!defined($valueA));
      $pkg->maccess_formatfce($tMM,$value,$maccess_formatfce,$datarow);
   }
   elsif($value=~/^<xhtml>/) {
      $tMM->aNchilds_fromStringHTML($value);
   }
   else {
      $tMM->aT(undef,{},$value);
   }
   # atribut value je zde kvuli update vystupum; je pritomen jen za predpokladu, ze doslo k nejake konverzi hodnoty (napriklad u typu dateTime) a ja chci v atributu zachovat hodnotu puvodni vracenou SQL servrem
   if(defined($valueA)) {
      $tMM->sA("value",$valueA);
   }
   # atribut type, je zde jen kvuli predchozimu Maccessu a mel vyuziti pri formatovani xml na strane klienta MS IE, dnes zatim jiz nema opodstatneni
   if($type) {
      $tMM->sA("dt:dt",$type);
   }
   # atribut style
   if($style) {
      $tMM->sA("style",$style);
   }
   # atribut href
   if($href) {
      $tMM->sA("href",$href);
   }
   # atribut hidden (u jednotlivych znacek se uvadi jen u znacky "rows_heading")
   if($tagname eq "rows_heading" and $$cols[$col]{hidden}) {
      $tMM->sA("hidden",$$cols[$col]{hidden});
   }
}




#-============= UPDATE funkce formularove ====================================-#
#--------------- Fce generujici aktualizacni formular
sub query_print_update {
   my ($pkg_p, $bodyO,$tableM,$outputdev,$param,$id,$idorig)=@_;
   # pridani radku s prazdnymi hodnotami u update_add - mohlo by se take zde treba vyplnovat necim preddefinovanym
   if($$param{query_type} eq "update_add") {
      # pokud je nejaky zaznam vracen tak si mohu neco zapamatovat, vyuzivam pro ziskani informace, zda se do nejakeho sloupce nezapisuji html data
      my @xhtmlA;
      if(my ($rowsM)=$tableM->findnodes('rows')) {
         my $position=0;
         foreach my $colR ($tableM->findnodes('rows//row/col')) {
            ++$position;
            if($colR->findnodes('xhtml')) {
               $xhtmlA[$position]=1;
            }
         }
         $tableM->removeChild($rowsM);
      }
      # zacinam na zelene louce
      my $rowsM=$tableM->aT("rows");
      my $rowM=$rowsM->aT(
         "row",
         {
            chosen => "0"
         }
      );
      my $position=0;
      foreach my $colC ($tableM->findnodes('cols/col')) {
         ++$position;
         if($xhtmlA[$position]) {
            $pkg->aTm(
               $rowM,
               "col",
               0,
               [ { value => "<xhtml></xhtml>" } ],
               [ { type => $colC->findvalue('@type') } ]
            );
         }
         else {
            $pkg->aTm(
               $rowM,
               "col",
               0,
               [ { value => undef } ],
               [ { type => $colC->findvalue('@type') } ]
            );
         }
      }
   }
   # naplneni hashe $polozky pro hash, ktery slouzi pak k definici formulare
   my ($maybeupdate,$polozky)=$pkg->update_form_def($tableM);
   my ($formO,$divO)=Form->form_val_list($bodyO,$polozky,[],"post");
   if($maybeupdate) {
      $divO->aT(
         "input",
         {
            name => "id",
            type => "hidden",
            value => $id
         }
      );
      $divO->aT(
         "input",
         {
            name => "idorig",
            type => "hidden",
            value => $idorig
         }
      );
      $pkg->update_buttonsT($formO,$$param{query_type});
   }
   my $href;
   # update
   if($$param{query_type} eq "update") {
      $href=$pkg->href_do($idorig);
   }
   # update_add
   else {
      $href=$pkg->href_do($id);
      $href.="&query_type=";
      $href.=encode_uri("xml_table");
      $href.="&limit_records=";
      $href.=encode_uri("0,100");
   }
   $bodyO->aT("p")->aT("a", { href => $href } , translate("view table"));
   ### good save from htmlarea
   # zaznamenani do $polozky do $param pro pozdejsi uziti
   foreach my $field (keys %{$polozky}) {
      if($field!~/^_/) {
         $$polozky{$field}{value}=undef; # nutne, jelikoz hodnoty mohou odkazovat na $xhtml a to je spatne ulozitelne a pak to blbne
      }
   }
   $$param{fields_update_form}=$polozky;
   # zaznamenani udaju v param do session - znovu, jelikoz jsem doplnoval $$param{fields_update_form}
   $pkg->sessionwrite($id,$param);
}

#--------------- Vutvoreni promene $polozky
sub update_form_def {
   my ($pkg_p, $tableM)=@_;
   my $maybeupdate=0;
   my $polozky;
   my $position=0;
   foreach my $colC ($tableM->findnodes('cols/col')) {
      ++$position; # Poznamka: moznost ziskani $position pomoci XPath: $position=$colC->findvalue('count(../preceding-sibling::node())+1');
      my ($colR)=$tableM->findnodes('rows//row/col[position()="'.$position.'"]');
      my $sqlname=$colC->findvalue('@sqlname');
      my $sqljoin=$colC->findvalue('@sqljoin');
      my $prikey=$colC->findvalue('@prikey');
      my $update=$colC->findvalue('@update');
      my $sqljoinexist=$tableM->findvalue('cols/col[attribute::sqlname="'.$sqljoin.'"]/@sqlname');
      my $noupdatejoinfrom=$tableM->findvalue('cols/col[attribute::sqljoin="'.$sqlname.'" and attribute::update!=1]/@sqlname');
      my $value;
      my $xhtmlO;
      if(my $valueA=$colR->findvalue('@value')) {
         $value=$valueA;
      }
      else {
         ($xhtmlO)=$colR->findnodes('xhtml');
         if($xhtmlO) {
            bless($xhtmlO,"MyXML");
            $value=$xhtmlO;
         }
         else {
            $value=$colR->findvalue('.');
         }
      }
      if(length($sqlname)) {
         #- primarni klice ($priname)
         if($prikey and $update) {
            my $priname="PRI.".$sqlname;
            $$polozky{$priname}{value}=$value;
            $$polozky{$priname}{type}="hidden";
            push(@{$$polozky{_layout}[0][0]{names}}, ($priname));
         }
         #- napojeni hodnot na jine hodnoty - plynouci ze spojovaciho vyrazu tabulek
         if($sqljoin and $sqljoinexist) {
            my $joinname="JOIN.".$sqlname;
            $$polozky{$joinname}{type}="hidden";
            $$polozky{$joinname}{value}=$sqljoin;
            push(@{$$polozky{_layout}[0][0]{names}}, ($joinname));
         }
         if($sqljoin and !$sqljoinexist and $update) {
            my $joinname="JOIN.".$sqljoin;
            $$polozky{$joinname}{type}="hidden";
            $$polozky{$joinname}{value}=$sqlname;
            push(@{$$polozky{_layout}[0][0]{names}}, ($joinname));
         }
      }
      #- samotna prace nad hodnotou ($name)
      my $name;
      if(length($sqlname)) {
         $name=$sqlname;
      }
      else {
         $name=$position;
      }
      $$polozky{$name}{print}=$colC->findvalue('@name');
      $$polozky{$name}{pri}=$prikey;
      $$polozky{$name}{value}=$value;
      $$polozky{$name}{style}=$colR->findvalue('@style');
      $$polozky{$name}{title}=$colR->findvalue('@title');
      # jde o fyzickou polozku z tabulky, kterou mohu aktualizovat
      if(
         length($sqlname) and
         (
            (!$sqljoin and !$noupdatejoinfrom and $update) or
            ($sqljoin and !$sqljoinexist and $update)
         )
      ) {
         $maybeupdate=1;
         # jde o memo polozku
         if(
            $colC->findvalue('@sqltype') eq "tinytext" or
            $colC->findvalue('@sqltype') eq "text" or
            $colC->findvalue('@sqltype') eq "mediumtext" or
            $colC->findvalue('@sqltype') eq "longtext"
         ) {
            if($xhtmlO) {
               $$polozky{$name}{type}="htmlarea";
               $$polozky{$name}{cols}="65";
               $$polozky{$name}{rows}="11";
               $$polozky{$name}{style}.="width: 580px; height: 300px;";
            }
            else {
               $$polozky{$name}{type}="textarea";
               $$polozky{$name}{cols}="65";
               $$polozky{$name}{rows}="6";
            }
         }
         # nejde o memo polozku
         else {
            $$polozky{$name}{type}="text";
            $$polozky{$name}{size}=$colC->findvalue('@length');
            if($$polozky{$name}{size}>65) {
               $$polozky{$name}{width}=65;
            }
            else {
               $$polozky{$name}{width}=$$polozky{$name}{size};
            }
         }
         push(@{$$polozky{_layout}[0][0]{names}}, ($name));
      }
      # jde o vypoctovou polozku, ktera nema v tabulce fyzicky ekvivalent a nebo jde o fyzickou polozku z tabulky, kterou nemohu aktualizovat
      elsif(
         !length($sqlname) or
         !$sqljoin or
         !$sqljoinexist
      ) {
         $$polozky{$name}{type}="show";
         push(@{$$polozky{_layout}[0][0]{names}}, ($name));
      }
   }
   return ($maybeupdate,$polozky);
}

#--------------- Tisk cudliku u update dotazu
sub update_buttonsT {
   my ($pkg_p, $formO,$query_type)=@_;
   my $pO=$formO->aT("p");
   if($query_type eq "update") {
      $pO->aT(
         "input",
         {
            name => "submit_update",
            type => "submit",
            value => translate("Update"),
            style => "background-color: #33AA33;"
         }
      );
      $pO->aT(undef,{}," ");
      $pO->aT(
         "input",
         {
            name => "submit_add",
            type => "submit",
            value => translate("Add")
         }
      );
      $pO->aT(undef,{}," ");
      $pO->aT(
         "input",
         {
            name => "submit_delete",
            type => "submit",
            value => translate("Delete")
         }
      );
   }
   else {
      $pO->aT(
         "input",
         {
            name => "submit_add",
            type => "submit",
            value => translate("Add"),
            style => "background-color: #33AA33;"
         }
      );
   }   
}




#-============= UPDATE funkce aktualizacni ===================================-#
#--------------- Vraci polozky tabulek ziskane z formulare form_update --------#
sub get_col_update {
   my ($pkg_p, $query)=@_;
   my (@tables_fullname,%col_val,%pri_val);
   # vracene hodnoty - %col_val
   foreach my $key ($query->param) {
      if($key=~/^([^\.]+)\.([^\.]+)\.([^\.]+)$/) {
         my $database=$1;
         my $table=$2;
         my $col=$3;
         $col_val{$database.".".$table}{$col}=$query->param($key);
         #- z formulare muze mit vstup zadany rucne ruznou datovou podobu - proto provadim jeste nejake konverze
         $col_val{$database.".".$table}{$col}=$pkg->correct_input($pkg->field_sqltype_by_col_fullname($database.".".$table.".".$col),$col_val{$database.".".$table}{$col});
         if(!grep(/^\Q$database.$table\E$/,@tables_fullname)) {
            push(@tables_fullname,$database.".".$table);
         }
      }
   }
   foreach my $key ($query->param) {
      if($key=~/^JOIN\.([^\.]+)\.([^\.]+)\.([^\.]+)$/) {
         my $database=$1;
         my $table=$2;
         my $col=$3;
         my $coljoin_fullname=$query->param($key);
         if($coljoin_fullname=~/^([^\.]+)\.([^\.]+)\.([^\.]+)$/) {
            my $databasejoin=$1;
            my $tablejoin=$2;
            my $coljoin=$3;
            $col_val{$database.".".$table}{$col}=$col_val{$databasejoin.".".$tablejoin}{$coljoin};
            if(!grep(/^\Q$database.$table\E$/,@tables_fullname)) {
               push(@tables_fullname,$database.".".$table);
            }
         }
      }
   }
   # vracene hodnoty primarnich klicu - %pri_val
   my @pris;
   foreach my $key ($query->param) {
      if($key=~/^PRI\.([^\.]+)\.([^\.]+)\.([^\.]+)$/) {
         my $database=$1;
         my $table=$2;
         my $col=$3;
         if(!grep(/^\Q$database.$table.$col\E$/,@pris)) {
            push(@pris,$database.".".$table.".".$col);
         }
         $pri_val{$database.".".$table}{$col}=$query->param($key);
         if(!grep(/^\Q$database.$table\E$/,@tables_fullname)) {
            push(@tables_fullname,$database.".".$table);
         }
      }
   }
   return (\@tables_fullname,\%col_val,\%pri_val);
}

#- z formulare muze mit vstup zadany rucne ruznou datovou podobu - proto provadim jeste nejake konverze
sub correct_input {
   my ($pkg_p,$data_type_string,$col_val)=@_;
   if($data_type_string eq "date") {
      $col_val=Maccess_query_string->correct_date($col_val);
   }
   elsif($data_type_string eq "datetime") {
      if($col_val=~/^(.+)[ T](.+)$/i) {
         my $date=$1;
         my $time=$2;
         $date=Maccess_query_string->correct_date($date);
         $col_val="$date $time";
      }
   }
   elsif($data_type_string eq "timestamp") {
      if($col_val=~/^(.+)[ T](.+)$/i) {
         my $date=$1;
         my $time=$2;
         $date=Maccess_query_string->correct_date($date);
         $col_val="$date $time";
      }
      $col_val=~s/[^\d]//gi;
      $col_val= $col_val . ("0" x (14-length($col_val)));
   }
   return $col_val;
}

#--------------- Funkce zjistuje, ktere tabulky mohou byt aktualizovany (znam u nich vsecky primarni klice) a to ze seznamu primarnich klicu
sub tables_maybeupdate {
   my ($pkg_p,@pris)=@_;
   # zjisteni vsech tabulek z kterych jsou primarni klice
   my @tables_fullname_pom;
   foreach my $pri (@pris) {
      $pri=~/^([^\.]+)\.([^\.]+)\.([^\.]+)$/;
      my $table_fullname="$1.$2";
      if(!grep(/^\Q$table_fullname\E$/,@tables_fullname_pom)) {
         push(@tables_fullname_pom,$table_fullname);
      }
   }
   # zjisteni tabulek, ktere je mozno aktualizovat - znam jejich primarni klice
   my @tables_fullname;
   foreach my $table_fullname (@tables_fullname_pom) {
      my $push_table=1;
      foreach my $pri (Maccess_query_string->primary_keys($table_fullname)) {
         if(!grep(/^\Q$pri\E$/,@pris)) {
            $push_table=0;
            last;
         }
      }
      if($push_table) {
         push(@tables_fullname,$table_fullname);
      }
   }
   return @tables_fullname;
}

#--------------- V tuto chvili ma fce zajistit jen dobre ukladani z html editorku
### good save from htmlarea
sub check_fields_update_form {
   my ($pkg_p, $query,$param)=@_;
   my $polozky=$$param{fields_update_form};
   foreach my $field (keys %{$polozky}) {
      if($field!~/^_/) {
         if($$polozky{$field}{type} eq "htmlarea") {
            my $value=$query->param($field);
            $query->param(-name=>$field,-value=>'<xhtml>'.$value.'</xhtml>');
         }
      }
   }
   return $query;
}

#--------------- Uprava zaznamu - Uprav -----------------#
sub update_rec {
   my ($pkg_p, $query)=@_;
   my %status;
   my ($tables_fullname,$col_val,$pri_val)=$pkg->get_col_update($query);
   # vyhledej zaznam u kazde tabulky a zjisit zda doslo ke zmenam pokud ano tak aktualizuj, ale jen ty sloupce co byly zmeneny (uzivatel muze mit pravo jen na zmenu nekterych sloupcu)
   foreach my $table_fullname (@$tables_fullname) {
      my @cols_update; # pole kde budou jmena sloupcu urcene k aktualizaci
      my @cols=keys %{$$col_val{$table_fullname}};
      my @pris=keys %{$$pri_val{$table_fullname}};
      my %query_hash;
      $query_hash{COLS}=join(",",@cols);
      $query_hash{FROM}=$table_fullname;
      $query_hash{WHERE}= join("=? and ",@pris) . "=?";
      $query_hash{LIMIT}="1";
      my $query_string=Maccess_query_string->query_hash_to_query_string(\%query_hash);
      my $sth=$dbh_usr->prepare($query_string);
      my $placeholder=1;
      foreach my $pri (@pris) {
         $sth->bind_param($placeholder, $$pri_val{$table_fullname}{$pri});
         ++$placeholder;
      }
      my $rv=$sth->execute;
      my $ref=$sth->fetchrow_arrayref;
      my $col_number=0;
      foreach my $col (@cols) {
         if($$ref[$col_number] ne $$col_val{$table_fullname}{$col}) {
            push(@cols_update,$col);
         }
         ++$col_number;
      }
      my $rc=$sth->finish;
      #- aktualizace
      if(@cols_update) {
         my $query_string="UPDATE $table_fullname SET ";
         $query_string.= join("=?,",@cols_update) . "=?";
         $query_string.= " WHERE ";
         $query_string.= join("=? and ",@pris) . "=?";
         my $sth=$dbh_usr->prepare($query_string);
         my $placeholder=1;
         foreach my $col (@cols_update) {
            if(0) {
               $sth->bind_param($placeholder, $$col_val{$table_fullname}{$col});
            }
            else {
               $sth->bind_param($placeholder, $$col_val{$table_fullname}{$col});
            }
            ++$placeholder;
         }
         foreach my $pri (@pris) {
            $sth->bind_param($placeholder, $$pri_val{$table_fullname}{$pri});
            ++$placeholder;
         }
         eval {
            local $SIG{__WARN__}=sub { $status{$table_fullname}=$_[0]; };
            my $rv=$sth->execute;
            if(defined($rv)) {
               $status{$table_fullname}="ok";
            }
         };
         my $rc=$sth->finish;
      }
   }
   return \%status;
}

#--------------- Uprava zaznamu - Pridej -----------------#
sub add_rec {
   my ($pkg_p, $query)=@_;
   my %status;
   my ($tables_fullname,$col_val,$pri_val)=$pkg->get_col_update($query);
   foreach my $table_fullname (@$tables_fullname) {
      my @cols_update=keys %{$$col_val{$table_fullname}};
      my $query_string="INSERT $table_fullname SET ";
      $query_string.= join("=?,",@cols_update) . "=?";
      my $sth=$dbh_usr->prepare($query_string);
      my $placeholder=1;
      foreach my $col (@cols_update) {
         $sth->bind_param($placeholder, $$col_val{$table_fullname}{$col});
         ++$placeholder;
      }
      eval {
         local $SIG{__WARN__}=sub { $status{$table_fullname}=$_[0]; };
         my $rv=$sth->execute;
         if(defined($rv)) {
            $status{$table_fullname}="ok";
         }
      };
      my $rc=$sth->finish;
   }
   return \%status;
}

#--------------- Uprava zaznamu - Smaz -----------------#
sub delete_rec {
   my ($pkg_p, $query)=@_;
   my %status;
   my ($tables_fullname,$col_val,$pri_val)=$pkg->get_col_update($query);
   foreach my $table_fullname (@$tables_fullname) {
      my @pris=keys %{$$pri_val{$table_fullname}};
      my $query_string="DELETE FROM $table_fullname";
      $query_string.= " WHERE ";
      $query_string.= join("=? and ",@pris) . "=?";
      my $sth=$dbh_usr->prepare($query_string);
      my $placeholder=1;
      foreach my $pri (@pris) {
         $sth->bind_param($placeholder, $$pri_val{$table_fullname}{$pri});
         ++$placeholder;
      }
      eval {
         local $SIG{__WARN__}=sub { $status{$table_fullname}=$_[0]; };
         my $rv=$sth->execute;
         if(defined($rv)) {
            $status{$table_fullname}="ok";
         }
      };
      my $rc=$sth->finish;
   }
   return \%status;
}

#--------------- Uprava zaznamu - Zobrazeni vysledku -----------------#
sub print_update_status {
   my ($pkg_p,$status,$type,$param)=@_;
   my $bad=grep(!/^\Qok\E$/,values(%$status));
   my $query=MyCGI->cgisession;
   my $href_redirect;
   my $href;
   # update
   # update by mel puvodne byt vzdy vyvolan z nejakeho tabulkoveho vystupu dotazu
   # a tak nabizim link pro navrat zpet
   if($$param{query_type} eq "update") {
      $href=$pkg->href_do($query->param('idorig'));
   }
   # update_add
   # u update_add puvodni dotaz pri prvnim spusteni neexistuje
   else {
      $href=$pkg->href_do($query->param('id'));
   }
   if($bad) {
      my @types=(translate("Update"),translate("Add"),translate("Delete"));
      my $idcs=$query->param('_idcs');
      my $outputdev=$pkg->outputdev($query);
      delete($$outputdev{bezpre});
      my ($docO,$htmlO,$bodyO)=$pkg->creahtml(
         $outputdev,
         $idcs,
         {
            h1 => $$param{query_name},
            query_id => $$param{query_id},
            folder => $$param{folder}
         }
      );
      $pkg->outputvar($bodyO,$$param{variables},$outputdev);
      my $ulO=$bodyO->aT("ul");
      foreach my $table_fullname (keys %$status) {
         my $liO=$ulO->aT("li", {}, $types[$type] . " " . $table_fullname . " - ");
         if($$status{$table_fullname} eq "ok") {
            $liO->aT("span", { style => "color: green;" }, translate("ok"));
         }
         else {
            $liO->aT("span", { style => "color: red;" }, translate("error"));
            $liO->aT("br");
            $liO->aT("b", {}, $$status{$table_fullname});
         }
      }
      $bodyO->aT("p")->aT("a", { href => $href } , translate("continue"));
      Output->outputprint($docO,$outputdev);
   }
   else {
      $href_redirect=$href;
   }
   return $href_redirect;
}

#--------------- Provedeni akcniho query_string ---------#
sub do_action_query_string {
   my ($pkg_p,$database,$query_string)=@_;
   my $sth=$dbh_usr->prepare($query_string);
   $sth->execute;
   $sth->finish;
}




#-============= DEKLARACE RUZNYCH FUNKCI ======================================-#
#--------------- Promena ve formulari generujici padaci menu z sql dotazu ------#
sub selectT_menu_q {
   my ($pkg_p, $query_string)=@_;
   my $selectmenu;
   my $query_hash=Maccess_query_string->query_string_to_query_hash($query_string);
   if(!length($$query_hash{FROM})) {
      $selectmenu=undef;
      my $sth=$dbh_usr->prepare($query_string);
      $sth->execute;
      my $ref=$sth->fetchrow_arrayref;
      $selectmenu=$$ref[0];
      $sth->finish;
   }
   else {
      $selectmenu=[];
      if(!length($$query_hash{LIMIT})) {
         $$query_hash{LIMIT}=$pkg->dim->{limit};
      }
      $query_string=Maccess_query_string->query_hash_to_query_string($query_hash);
      my $sth=$dbh_usr->prepare($query_string);
      $sth->execute;
      while(my $ref=$sth->fetchrow_arrayref) {
         ### if nize je blba berlicka, ale za soucasne situace neni mozne nic jineho delat (MyCGI to zase dava do poradku)
         if($$ref[0] eq "") {
            $$ref[0]="undef";
         }
         push(@{$selectmenu},[$$ref[0],$$ref[1]]);
      }
      $sth->finish;
   }
   return $selectmenu;
}

#--------------- Promena ve formulari generujici padaci menu -------------------#
sub selectT_menu {
   my ($pkg_p, $values)=@_;
   my $selectmenu=[];
   my @values=split(/\|/,$values);
   for(my $i=0; $i<=$#values; $i+=2) {
      ### if nize je blba berlicka, ale za soucasne situace neni mozne nic jineho delat (MyCGI to zase dava do poradku)
      if($values[$i] eq "") {
         $values[$i]="undef";
      }
      push(@{$selectmenu},[$values[$i],$values[$i+1]]);
   }
   return $selectmenu;
}

#--------------- fce seradi pointer na pole pomoci fce sort_array_ival a vrati pointer na nove serazene pole
sub sort_array {
   my ($pkg_p, $data,$type,$ascdesc)=@_;
   my @ival=$pkg->sort_array_ival($data,$type,$ascdesc);
   my $data_sort;
   for(my $row=0;$row<=$#{$data};++$row) {
      $$data_sort[$row]=$$data[$ival[$row]];
   }
   return $data_sort;
}

#--------------- fce seradi pointer na pole pomoci SQL databaze a to dle prani i numericky a vrati prevodni pole, jak se maji zmenit indexy, aby vse bylo serazeno
# od uziti databaze si slibuji zejmena spravne razeni dle defaultniho jazyka na SQL databazi (mozna by bylo dobre pozdeji umet jazyk nastavit)
sub sort_array_ival {
   my ($pkg_p, $data,$type,$ascdesc)=@_;
   my @ival;
   # zjednoduseni v razeni:
   # - vsechna cisla povazuji za double
   # - vse ostatni za varchar(255)
   # Pozn: dokonale by bylo vytvorit presne spravny typ sloupce
   #       jakeho jsou puvodni data (pozdeji i s definici jazyka);
   #       bohuzel nemam prostredky jak to u vypoctenych sloupcu
   #       zjistit
   my $val_type;
   if($type eq "number") {
      $val_type="double";
   }
   else {
      $val_type="varchar(255) binary";
   }
   my $sth_sort=$dbh_sys->prepare(q{
      create temporary table
         Sort
         (
            ival int not null,
            val } . $val_type . q{,
            primary key (ival)
         )
   });
   $sth_sort->execute;
   my $sth_sort=$dbh_sys->prepare(q{
      insert into
         Sort
      values
         (?,?)
   });
   for(my $row=0;$row<=$#{$data};++$row) {
      $sth_sort->execute($row,$$data[$row]);
   }
   my $sth_sort=$dbh_sys->prepare(q{
      select
         Sort.*
      from
         Sort
      order by
         val } . $ascdesc . q{
   });
   $sth_sort->execute;
   while(my $ref_sort=$sth_sort->fetchrow_hashref) {
      push(@ival,$$ref_sort{ival});
   }
   $sth_sort->finish;
   my $sth_sort=$dbh_sys->prepare(q{
      drop table
         Sort
   });
   $sth_sort->execute;
   return @ival;
}

# obecne pouziti 2 funkci nize:
# query_mrizka -> encode_maccess;  sloupce vracene MySQL -> decode_maccess
#--------------- Funkce kodujici znaky nazvu sloupcu vracenych MySQL do "maccessovskeho" opisu pouzitelneho pro MySQL a take XML znacky
sub encode_maccess {
   my ($pkg_pom, $str)=@_;
   $str=~s/^([0-9])/"_x".unpack("H2",$1)."_"/e;  # v XML nemuze zacinat znacka cislici - proto pokud retezec zacina cislici je nutne encodovat i ji (muze klidne zacinat znakem _)
   $str=~s/([\x20-\x2F\x3A-\x40\x5B-\x5E\x60\x7B-\x7E])/"_x".unpack("H2",$1)."_"/ge;  # ASCII tabulka: http://members.tripod.com/~plangford/index.html
   return $str;
}
#--------------- Funkce dekodujici znaky sloupcu ze znacek XML pro zjisteni skutecneho nazvu sloupce
sub decode_maccess {
   my ($pkg_pom, $str)=@_;
   $str=~s/_x([\da-f]{2,3})_/pack("H2",$1)/ge;
   return $str;
}
#------- Funkce zakoduje uri zpusobem napred znak % a pak znaky , | - vhodne pro listy, kde jako oddelovace mezi klicem a hodnou pouzivas "," a mezi pary "|" (klice i hodnoty musis zakodovat a pozdeji zvlast dekodovat)
sub encode_list {
   my ($pkg_pom, $str)=@_;
   $str=~s/([\%])/"%".unpack("H2",$1)/ge;
   $str=~s/([\,\|])/"%".unpack("H2",$1)/ge;
   return $str;
}
#------- Funkce dekoduje z uri opisu znaky , | %
sub decode_list {
   my ($pkg_pom, $str)=@_;
   $str=~s/\%2C/\,/gi;
   $str=~s/\%7C/\|/gi;
   $str=~s/\%25/\%/gi;
   return $str;
}
#------- Funkce likviduje v retezci vsechny znacky <xhtml> a </xhtml>
sub drop_xhtml {
   my ($pkg_pom, $str)=@_;
   $str=~s/<(\/{0,1})xhtml>//gi;
   return $str;
}
#------- Funkce upravuje retezec, aby se lepe hodil pro popisek grafu
sub graph_popis {
   my ($pkg_pom, $str)=@_;
   $str=substr($pkg->drop_xhtml($str),0,30);
   return $str;
}
#------- Funkce zajistujici prekodovani znaku mezi znakovymi sadamy
sub recode_charset {
   my ($pkg_p, $str_s,$charset_s,$charset_t)=@_;
   my $str_t;
   # jmena znakovych sad pouzivanych v knihovne Unicode::Map8 jsou jina a musi se pro ni "prelozit"
   my %charset_pkgUnicode=Maccess_locale->charset_pkgUnicode;
   if(!$charset_pkgUnicode{$charset_s}) {
      die "Unknow charset $charset_s, when recode string $str_s to charset $charset_t";
   }
   $charset_s=$charset_pkgUnicode{$charset_s};
   $charset_t=$charset_pkgUnicode{$charset_t};
   if($charset_t eq "UTF-8") {
      my $map_s=Unicode::Map8->new($charset_s) || die $charset_s;
      $str_t=$map_s->to16($str_s); # prevod do UTF-16
      # neni testovano
      $str_t=Unicode::String::utf16($str_t); # "oznameni", ze retezec je v UTF-16
      $str_t=$str_t->utf8; # prevod do UTF-8
   }
   elsif($charset_s eq "UTF-8") {
      my $map_t=Unicode::Map8->new($charset_t) || die $charset_t;
      $str_s=Unicode::String::utf8($str_s); # "oznameni", ze retezec je v UTF-8
      $str_s=$str_s->utf16; # prevod do UTF-16
      $str_t=$map_t->to8($str_s); # konverze z UTF-16 do prislusne 8bitove sady
   }
   else {
      my $map_s=Unicode::Map8->new($charset_s) || die $charset_s;
      my $map_t=Unicode::Map8->new($charset_t) || die $charset_t;
      $str_t=$map_s->recode8($map_t,$str_s);
   }
   return $str_t;
}

#--------------- Funkce vraci seznam klicu a hodnot, ktere se podili na grupovani a je potreba je pouzit k odgrupovani
sub ungroup_by {
   my ($pkg_p, $cols_group,$names_hash)=@_;
   my $ungroup_by;
   foreach my $col_group (@$cols_group) {
      $ungroup_by.= $pkg->encode_list($col_group) . "," . $pkg->encode_list($$names_hash{$col_group}) . "|";
   }
   chop($ungroup_by);
   return $ungroup_by;
}

#--------------- Cast linku pouzivaneho v Maccess vystupech pro ungroup, update, limit
sub href_do {
   my ($pkg_p, $id)=@_;
   my $href;
   if(defined($pkg->query_id_system)) {
      $href=My->okruh."?";
   }
   else {
      $href="do.phtml?";
   }
   $href.="id=";
   $href.=encode_uri($id);
   return $href;
}

#--------------- Link pro Ungroup
sub href_ungroup_by {
   my ($pkg_p, $id,$cols_group,$names_hash)=@_;
   my $href=$pkg->href_do($id);
   $href.="&ungroup_by=";
   my $ungroup_by=$pkg->ungroup_by($cols_group,$names_hash);
   $href.=encode_uri($ungroup_by);
   return $href;
}

#--------------- Link pro Update
sub href_update_record {
   my ($pkg_p, $id,$record)=@_;
   my $href=$pkg->href_do($id);
   $href.="&update_record=";
   $href.=encode_uri($record);
   return $href;
}

#--------------- Link pro Order
sub href_order_by {
   my ($pkg_p, $id,$cols_order)=@_;
   my $href=$pkg->href_do($id);
   $href.="&order_by=";
   $href.=encode_uri($cols_order);
   return $href;
}

#--------------- Link pro "Dalsich X radku"
sub href_limit_records {
   my ($pkg_p, $id,$limit)=@_;
   my $href=$pkg->href_do($id);
   $href.="&limit_records=";
   $href.=encode_uri(($$limit[0]+$$limit[1]) . "," . $$limit[1]);
   return $href;
}

#--------------- Link pro Specialni nastaveni tabilky
sub href_spec_xml_table {
   my ($pkg_p, $id)=@_;
   my $href;
   $href="query_spec_xml_table.phtml?";
   $href.="id=";
   $href.=encode_uri($id);
   return $href;
}

#--------------- Link pro Specialni nastaveni formulare
sub href_spec_form {
   my ($pkg_p, $id)=@_;
   my $href;
   $href="query_spec_form.phtml?";
   $href.="id=";
   $href.=encode_uri($id);
   return $href;
}

#--------------- Tisk chybove hlasky pri selhani spusteni dotazu v Maccessu
sub print_error_syntax {
   my ($pkg_p, $param,$error)=@_;
   my $query=MyCGI->cgisession;
   my $idcs=$query->param('_idcs');
   my $outputdev=$pkg->outputdev($query);
   my ($docO,$htmlO,$bodyO)=$pkg->creahtml(
      $outputdev,
      $idcs,
      {
         h1 => translate("Syntax error in query :0.",$$param{query_name}),
         query_id => $$param{query_id},
         folder => $$param{folder}
      }
   );
   my $pO=$bodyO->aT("p");
   $pO->aT("b", {}, translate("SQL command:"));
   $bodyO->aT("p", {}, $$param{query_string});
   my $pO=$bodyO->aT("p");
   $pO->aT("b", {}, translate("Error returned by SQL server:"));
   $bodyO->aT("p", {}, $error);
   Output->outputprint($docO,$outputdev);
   return 1;
}

#--------------- newXML outputdev
sub outputdev {
   my ($pkg_p, $query)=@_;
   my $outputdev=My->outputdev($query);
   # no nevim zda je toto ok - nema se nastaveni znakove sady brat z definice zarizeni? skoro vznika otakzka zda i jazyk nema byt bran z definice zarizeni (niz takove tam zatim nemam a myslim, ze to tam byt asi nema)
   $$outputdev{cestina}=Maccess_locale->charset;
   $$outputdev{bezpre}=1;
   return $outputdev;
}

#--------------- newXML Hlavicka a paticka HTML dokumentu
sub creahtml {
   my ($pkg_p, $outputdev,$idcs,$m)=@_;
   if(!defined($pkg->query_id_system)) {
      $$outputdev{patickahloubka}=1;
   }
   else {
      $$outputdev{patickahloubka}=999;
   }
   my ($docO,$htmlO,$bodyO);
   ($docO,$htmlO,$bodyO,$$outputdev{prvniradek})=MyXML->creahtml($outputdev,$idcs);
   # dodani spesl veci pro Maccess do hlavicky HTML dokumentu
   if(my ($headO)=$htmlO->findnodes('head')) {
      bless($headO,"MyXML");
      if(!defined($pkg->query_id_system)) {
         # zmena title
         if( my ($titleO)=$headO->findnodes('title') ) {
            bless($titleO,"MyXML");
            $headO->removeChild($titleO);
            $pkg->titleT($headO,$m);
         }
      }
   }
   # dalsi spesl veci
   if(!defined($pkg->query_id_system)) {
      # zmena h1
      my $h1O;
      if( ( ($h1O)=$htmlO->findnodes('body//h1') ) or ( ($h1O)=$htmlO->findnodes('body//span[@class="h1"]') ) ) {
         bless($h1O,"MyXML");
         if(my ($textO)=$h1O->firstChild) {
            if(ref($textO)) {
               bless($textO,"MyXML");
               $h1O->removeChild($textO);
            }
            $h1O->aT(undef,{}, $$m{h1} );
         }
      }
      # zmena paticky - posledni linky necht se ridi slozkou, kde se nachazim
      if(my ($divO)=$htmlO->findnodes('body//div[@class="footer"]')) {
         bless($divO,"MyXML");
         my $info_query;
         my $folder;
         if(length($$m{query_id})) {
            my $query_id=$$m{query_id};
            $info_query=$pkg->info_query($$m{query_id});
            $folder=$$info_query{folder_id};
         }
         elsif($$m{folder}>0) {
            $folder=$$m{folder};
         }
         my @supfolders=$pkg->supfolders($folder);
         my $poprve=1;
         foreach my $folder_pom (@supfolders,$folder) {
            if(!$poprve or length(My->okruh_nazev("/"))) {
               $divO->aT(undef,{}," | ");
            }
            my $info_folder_pom=$pkg->info_folder($folder_pom);
            $divO->aT(
               "a",
               {
                  class => "footer",
                  target => "_top",
                  href => "./?folder=" . encode_uri($$info_folder_pom{folder_id})
               },
               $$info_folder_pom{folder_name}
            );
            $poprve=0;
         }
         if($$info_query{_exist} and My->okruh=~/\/do\.phtml/) {
            my $info_query_group_top=$pkg->info_query_group_top($$info_query{query_id});
            my $href_string= "query_id=" . encode_uri($$info_query_group_top{query_id});
            $divO->aT(undef,{}," | ");
            $divO->aT(
               "a",
               {
                  class => "footer",
                  target => "_top",
                  href => "do.phtml?" . $href_string
               },
               $$info_query_group_top{query_name}
            );
         }
      }
   }
   return ($docO,$htmlO,$bodyO,$$outputdev{prvniradek});
}

#--------------- Vklada znacku title
sub titleT {
   my ($pkg_p, $tO,$m)=@_;
   $tO->aT("title",{}, $maccess_ver . " - " . $$m{h1} . ' (' . $pkg->get_remote_user . ' ' . My->datetime_now . ')' );
}

#--------------- Tisk hodnot promenych ve vystupu
sub outputvar {
   my ($pkg_p, $tO,$vars,$outputdev)=@_;
   if(defined($vars)) {
      my $outputvars;
      my $i=0;
      foreach my $var (@{$vars}) {
         my $polozka=$$var[2];
         if(defined($$polozka{print})) {
            $$outputvars[$i][0]=$$polozka{print}.":";
         }
         else {
            $$outputvars[$i][0]=$$var[0].":";
         }
         if(defined($$polozka{value})) {
            if(defined($$polozka{format})) { # kod prevzaty s Form.pm
               my ($pkg_ext,$format_fce)=split(/\-\>/,$$polozka{format});
               $$outputvars[$i][1]=$pkg_ext->$format_fce($$polozka{value});
            }
            elsif(defined($$polozka{selectmenu})) { # kod prevzaty s Form.pm
               my @selectmenu=Form->selectmenu_selectmenu($$polozka{selectmenu});
               foreach my $selectmenu (@selectmenu) {
                  if($$selectmenu[0] eq $$polozka{value}) {
                     $$outputvars[$i][1]=$$selectmenu[1];
                  }
               }
            }
            else {
               $$outputvars[$i][1]=$$polozka{value};
            }
         }
         else {
            $$outputvars[$i][1]=$$var[1];
         }
         ++$i;
      }
      Output->outputvar($tO,$outputvars,$outputdev);
   }
}

#-------------- Obdrzeni promenych z formulare
sub form_variables_get {
   my ($pkg_p, $polozky)=@_;
   my @variables;
   foreach my $name (Form->names($polozky)) {
      if($name=~/^VAR_(.+)$/) {
         push(@variables, [$1,$$polozky{$name}{value},$$polozky{$name}]);
      }
   }
   return \@variables;
}

## funkce nize by to chtelo nejak skonzistentnit a vse pekne pojmenovat

#-------------- Funkce urcujici XML typ polozky
# musi byt zalozena na data_type_number, jelikoz jsou vraceny sloupce, ktere jsou slouceninou ruznych sloupcu nebo po aplikaci funkci
sub field_type {
   my ($pkg_p,$data_type_number)=@_;
   my @type_info=$dbh_sys->type_info($data_type_number);
   my $data_type_string=$type_info[0]{TYPE_NAME};
   my $data_type_string_xml=$pkg->field_type_by_sqltype($data_type_string);
   return $data_type_string_xml;
}

#-------------- Funkce urcujici XML typ polozky na zaklade typu MySQL
sub field_type_by_sqltype {
   my ($pkg_p,$data_type_string)=@_;
   my %data_types_string_xml=(
      "tinyint" => "number",
      "smallint" => "number",
      "mediumint" => "number",
      "int" => "number",
      "integer" => "number",
      "bigint" => "number",
      "float" => "number",
      "double" => "number",
      "double precision" => "number",
      "real" => "number",
      "decimal" => "number",
      "numeric" => "number",
      "date" => "date",
      "datetime" => "dateTime",
      "timestamp" => "dateTime",   # podporuji jen timestamp ve tvaru YYYYMMDDHHMMSS
      "time" => "time"
   );
   my $data_type_string_xml="$data_types_string_xml{$data_type_string}";
   return $data_type_string_xml;
}

#-------------- Funkce urcujici XML typ polozky na zaklade celeho jmena sloupce
sub field_sqltype_by_col_fullname {
   my ($pkg_p,$col_fullname)=@_;
   my $field_def=$pkg->field_def($col_fullname);
   my $data_type_string=$$field_def{Type};
   $data_type_string=~s/\((\d+),{0,1}\d*\)$//;
   return $data_type_string;
}

#-------------- Funkce urcujici XML typ polozky
sub field_length {
   my ($pkg_p,$col_fullname)=@_;
   my $length;
   if(defined($col_fullname)) {
      my $field_def=$pkg->field_def($col_fullname);
      if($$field_def{Type}=~/\((\d+),{0,1}\d*\)/) {
         $length=$1;
      }
      elsif($$field_def{Type} eq "date") {
         $length=10;
      }
      elsif($$field_def{Type} eq "datetime") {
         $length=19;
      }
      elsif($$field_def{Type} eq "time") {
         $length=8;
      }
   }
   return $length;
}

#-------------- Funkce popisujici definici sloupce v tabulce
sub field_def {
   my ($pkg_p,$col_fullname)=@_;
   my ($database,$table,$col)=split(/\./,$col_fullname);
   my $sth=$dbh_sys->prepare(q{
      show
         columns
      from
         } . $database . q{.} . $table . q{ like ?
   });
   $sth->execute($col);
   my $ref=$sth->fetchrow_hashref;
   my $field_def=$ref;
   $sth->finish;
   return $field_def;
}

#-------------- Funkce popisujici definici sloupce v tabulce - vraci retezec
sub field_def_str {
   my ($pkg_p,$col_fullname)=@_;
   my $field_def_str;
   if($col_fullname!~/^([^\.]+)\.([^\.]+)\.(\*)$/) {
      my $field_def=$pkg->field_def($col_fullname);
      $field_def_str="$$field_def{Field} $$field_def{Type}";
      if(defined($$field_def{Default})) {
         $field_def_str.=" DEFAULT '$$field_def{Default}'";
      }
      if($$field_def{Null} ne "YES") {
         $field_def_str.=" NOT NULL";
      }
   }
   return $field_def_str;
}

#----- Funkce prevalcuje nektere hodnoty v param, pokud prisli primo GET ci POST metodou v pozadavku
sub update_param_by_GET_or_POST {
   my ($pkg_p, $param,$query)=@_;
   if(!defined($pkg->query_id_system)) {
      if(defined($query->param("link"))) {
         $$param{link}=$query->param("link");
      }
      if(defined($query->param("query_type"))) {
         $$param{query_type}=$query->param("query_type");
      }
   }
   return $param;
}

#-------------- Funkce urcujici typ vystupu v zavislosti na prohlizeci a typu dotazu
sub user_agent {
   my $user_agent;
   if($ENV{HTTP_USER_AGENT}=~/MSIE [56789]/ and not $ENV{HTTP_USER_AGENT}=~/Windows 3\./) {
      $user_agent="MSIE";
   }
   elsif($ENV{HTTP_USER_AGENT}=~/Gecko/) {
      $user_agent="MOZILLA";
   }
   else {
      $user_agent="OTHER";
   }
   return $user_agent;
}

#----------- Vrati uzivatele, ktery dotaz vytvoril
sub create_user_query {
   my ($pkg_p, $query_id)=@_;
   my $create_user;
   if(length($query_id)) {
      my $sth=$dbh_sys->prepare(q{
         select
            Query.create_user
         from
            Query
         where
            Query.query_id=?
      });
      $sth->execute($query_id);
      my $ref=$sth->fetchrow_hashref;
      $create_user=$$ref{create_user};
      $sth->finish;
   }
   else {
      $create_user=$maccess_power_user;
   }
   return $create_user;
}

#----------- Vrati uzivatele, ktery slozku vytvoril
sub create_user_folder {
   my ($pkg_p, $folder)=@_;
   my $create_user;
   if($folder>0) {
      my $sth=$dbh_sys->prepare(q{
         select
            Folder.create_user
         from
            Folder
         where
            Folder.folder_id=?
      });
      $sth->execute($folder);
      my $ref=$sth->fetchrow_hashref;
      $create_user=$$ref{create_user};
      $sth->finish;
   }
   else {
      $create_user=$maccess_power_user;
   }
   return $create_user;
}

#----------- Funkce slouzici k tisku padacich menu
sub optionT {
   my ($pkg_p, $tO,$option_val,$option_text,$option_class,$option_val_selected)=@_;
   for(my $i=0; $i<=$#$option_val; ++$i) {
      my $optionO=$tO->aT("option", { value => $$option_val[$i] }, $$option_text[$i]);
      if(defined($$option_class[$i])) {
         $optionO->sA("class", $$option_class[$i]);
      }
      if($option_val_selected eq $$option_val[$i]) {
         $optionO->sA("selected", "selected");
      }
   }
}

#----------- Funkce slouzici k tisku padaciho meny z vyberem typu zobrazeni
sub selectT_query_type {
   my ($pkg_p, $tO,$default)=@_;
   my $selectO=$tO->aT("select", { "name" => "query_type" });
   # Nektere druhy vystupu Maccessu jsou nyni zruseny,
   # jelikoz je zajistuje vystupni aparat v Output.pm.
   $pkg->optionT(
      $selectO,
      [
         #"auto",
         #"txt",
         #"txttable",
         #"table",
         "xml_table",
         "xml_raw",
         #"excel",
         #"file",
         "graph_bars",
         "graph_bars_sum",
         "graph_pie",
         "graph_area",
         "graph_lines",
         "graph_linespoints",
         "map_Europe",
         "update_add"
      ],
      [
         #translate("auto"),
         #translate("txt"),
         #translate("txttable"),
         #translate("table"),
         translate("table"), #translate("xml table"),
         translate("xml raw"),
         #translate("Excel 97"),
         #translate("file ~/tmp/maccess.txt"),
         translate("graph - bars"),
         translate("graph - bars sum"),
         translate("graph - pie"),
         translate("graph - area"),
         translate("graph - lines"),
         translate("graph - linespoints"),
         translate("map - Europe"),
         translate("add row")
      ],
      [],
      $default
   );
}





#-============= Funkce pro praci s Maccess sessions ===========================-#
sub sessionread {
   my ($pkg_p, $id)=@_;
   my $param=MySession->read($id);
   # test na ukradenou session u dotazu zakomponovanych do systemu
   if(defined($pkg->query_id_system)) {
      my $stolensession=1;
      my $info_query_ungroup=$pkg->info_query($pkg->query_id_system);
      while($$info_query_ungroup{_exist}) {
         if($$info_query_ungroup{query_id} eq $$param{query_id}) {
            $stolensession=0;
            last;
         }
         $info_query_ungroup=$pkg->info_query_ungroup($$info_query_ungroup{query_id});
      }
      if($stolensession) {
         die "Error: stolen session (query_id)";
      }
   }
   return $param;
}
sub sessionwrite {
   my ($pkg_p, $id,$param)=@_;
   $id=MySession->write($id,$param);
   return $id;
}




#-============= Test na spusteni dotazy integrovaneho do systemu ==============-#
sub query_id_system {
   my $query_id;
   if(My->okruh=~/\/do\.maccess~query_id=(\d+)/) {
      $query_id=$1;
   }
   return $query_id;
}




#-============= Funkce pro praci se slozkami ==================================-#
#--- Funkce vracejici seznam slozek
sub folders {
   my ($pkg_p, $folder)=@_;
   my (@folders);
   if($folder<=0) {
      $folder=0;
   }
   my $sth=$dbh_sys->prepare(q{
      select
         Folder.folder_id
      from
         Folder
      where
         Folder.supfolder_id= ?
      order by
         Folder.folder_name asc
   });
   $sth->execute($folder);
   while(my $ref=$sth->fetchrow_hashref) {
      push @folders, $$ref{folder_id};
   }
   $sth->finish;
   return @folders;
}
#--- Funkce vracejici cely strom slozek od zadane slozky
sub folders_tree {
   my ($pkg_p, $folder,$nofolder)=@_;
   my (@folders_tree);
   if($folder ne $nofolder) {
      push(@folders_tree,$folder);
      foreach my $foldertree ($pkg->folders($folder)) {
         push(@folders_tree,$pkg->folders_tree($foldertree,$nofolder));
      }
   }
   return @folders_tree;
}
#--- Funkce vracejici seznam slozek
sub folders_menu {
   my ($pkg_p, $folder,$nofolder)=@_;
   my (@folders_menu);
   my @supfolders=$pkg->supfolders($folder);
   foreach my $folderroot ($pkg->folders) {
      if($folderroot eq $supfolders[0]) {
         push(@folders_menu,$pkg->folders_tree($supfolders[0],$nofolder));
      }
      elsif($folderroot ne $nofolder) {
         push(@folders_menu,$folderroot);
         push(@folders_menu,$pkg->folders($folderroot));
      }
   }
   return @folders_menu;
}
#--- Funkce vracejici info o folderu na zaklade jeho id
sub info_folder {
   my ($pkg_p, $folder)=@_;
   my $ref;
   if($folder>0) {
      my $sth=$dbh_sys->prepare(q{
         select
            Folder.*
         from
            Folder
         where
            Folder.folder_id=?
      });
      $sth->execute($folder);
      $ref=$sth->fetchrow_hashref;
      $sth->finish;
      $$ref{_exist}=$sth->rows;
   }
   else {
      $ref={
         folder_id => 0,
         folder_name => 'Maccess',
         _exist => 1
      };
   }
   return $ref;
}
#--- Funkce vracejici info o folderu na zaklade jeho jmena a supfolder_id
sub info_folder_name {
   my ($pkg_p, $supfolder,$folder_name)=@_;
   my $ref;
   if($supfolder<=0) {
      $supfolder=0;
   }
   my $sth=$dbh_sys->prepare(q{
      select
         Folder.*
      from
         Folder
      where
         Folder.supfolder_id=? and
         Folder.folder_name=?
   });
   $sth->execute($supfolder,$folder_name);
   $ref=$sth->fetchrow_hashref;
   $sth->finish;
   $$ref{_exist}=$sth->rows;
   return $ref;
}
#--- Funkce vracejici vysi folder k zadanemu folderu
sub supfolder {
   my ($pkg_p, $folder)=@_;
   my $info_folder=$pkg->info_folder($folder);
   return $$info_folder{supfolder_id};
}
#--- Funkce vracejici vysi foldry k zadanemu folderu
sub supfolders {
   my ($pkg_p, $folder)=@_;
   my @supfolders;
   $folder=$pkg->supfolder($folder);
   while(defined($folder)) {
      push(@supfolders,$folder);
      $folder=$pkg->supfolder($folder);
   }
   return reverse(@supfolders);
}
#--- Funkce vracejici jak je prislusny folder hluboko v hiearchii slozek
sub folder_depth {
   my ($pkg_p, $folder)=@_;
   my $depth=0;
   my $info_supfolder=$pkg->info_folder($folder);
   while($$info_supfolder{_exist} and $$info_supfolder{folder_id}!=0 and $depth<255) {
      $info_supfolder=$pkg->info_folder($$info_supfolder{supfolder_id});
      ++$depth;
   }
   return $depth;
}


#-============= Funkce pro praci s dotazy =====================================-#
#--- Funkce vracejici seznam dotazu
sub queries {
   my ($pkg_p, $folder)=@_;
   my (@queries);
   if($folder>0) {
      my $dbh_sys=Maccess_db->dbh_sys;
      # dotaz nacitajici seznam dotazu v dane databazi a slozce
      my $sth=$dbh_sys->prepare(q{
         select
            Query.query_id
         from
            Query
         where
            Query.folder_id=? and
            Query.query_id_group=0
         order by
            Query.query_name asc,
            Query.query_id asc
      });
      $sth->execute($folder);
      while(my $ref=$sth->fetchrow_hashref) {
         push @queries, $$ref{query_id};
      }
      $sth->finish;
   }
   return @queries;
}
#--- Funkce vracejici info o dotazu na zaklade jeho id
sub info_query {
   my ($pkg_p, $query_id)=@_;
   my $sth=$dbh_sys->prepare(q{
      select
         Query.*
      from
         Query
      where
         Query.query_id=?
   });
   $sth->execute($query_id);
   my $ref=$sth->fetchrow_hashref;
   $sth->finish;
   $$ref{_exist}=$sth->rows;
   return $ref;
}
#--- Funkce vracejici info o dotazu na zaklade umisteni ve slozce a jeho jmena
sub info_query_name {
   my ($pkg_p, $folder,$query_name)=@_;
   my $sth=$dbh_sys->prepare(q{
      select
         Query.*
      from
         Query
      where
         Query.folder_id=? and
         Query.query_name=?
   });
   $sth->execute($folder,$query_name);
   my $ref=$sth->fetchrow_hashref;
   $sth->finish;
   $$ref{_exist}=$sth->rows;
   return $ref;
}
#--- Funkce vracejici info o ungroup dotazu na zaklade id "naddotazu"
sub info_query_ungroup {
   my ($pkg_p, $query_id)=@_;
   my $sth=$dbh_sys->prepare(q{
      select
         Query.*
      from
         Query
      where
         Query.query_id_group=?
   });
   $sth->execute($query_id);
   my $ref=$sth->fetchrow_hashref;
   $sth->finish;
   $$ref{_exist}=$sth->rows;
   return $ref;
}
#--- Funkce vracejici info o nejvysim group dotazu na zaklade id dotazu ci "poddotazu"
sub info_query_group_top {
   my ($pkg_p, $query_id)=@_;
   my $info_query_group_top;
   my $info_query_group=$pkg->info_query($query_id);
   while($$info_query_group{_exist}) {
      $info_query_group_top=$info_query_group;
      $info_query_group=$pkg->info_query($$info_query_group{query_id_group});
   }
   return $info_query_group_top;
}




#-====== Zakladni funkce Maccessu pro zjistovani informaci o DB a tabulkach ======-#
#-- Kdo se do Maccessu prihlasil
sub get_remote_user {
   my $remote_user=$ENV{REMOTE_USER};
   return $remote_user;
}

#-- S jakymi pravy ma dotaz v Maccessu bezet
sub get_maccess_user {
   my $maccess_user;
   if(defined(Maccess->query_id_system)) {
      my $create_user_query=$pkg->create_user_query(Maccess->query_id_system);
      my $info_okruh=My->okruh_info;
      if($create_user_query eq $$info_okruh{uzivzapis}) {
         $maccess_user=$create_user_query;
      }
      else {
         die "Error: get_maccess_user (okruh: " . $$info_okruh{uzivzapis} . "; query: " . $create_user_query . ")";
      }
   }
   else {
      $maccess_user=$pkg->get_remote_user;
   }
   return $maccess_user;
}

#-- Funkce vracejici pole se seznamem databazi na ktere ma uzivatel pravo
sub databases {
   my (@databases);
   # user - predpokladam, ze uzivatel ma v tabulce user jen jeden zaznam
   my $sth=$dbh_sys->prepare(q{
      select
         user.Select_priv
      from
         mysql.user
      where
         user.User=?
   });
   $sth->execute($pkg->get_remote_user);
   my $ref=$sth->fetchrow_arrayref;
   my $select_priv=$$ref[0];
   $sth->finish;
   # pokud ma uzivatel select pravo na vsechny DB
   if($select_priv eq "Y") {
      my $sth=$dbh_sys->prepare(q{
         show
            databases
      });
      $sth->execute;
      while(my $ref=$sth->fetchrow_arrayref) {
         push @databases, $$ref[0];
      }
      $sth->finish;
   }
   # pokud nema uzivatel select pravo na vsechny DB
   elsif($select_priv eq "N") {
      # db - neuvazuji mozne ruzne zaznamy uzivatele pro ruzne Host
      my $sth=$dbh_sys->prepare(q{
         select
            db.Db
         from
            mysql.db
         where
            db.User=?
      });
      $sth->execute($pkg->get_remote_user);
      while(my $ref=$sth->fetchrow_arrayref) {
         push @databases, $$ref[0] if(!grep(/^\Q$$ref[0]\E$/, @databases));
      }
      $sth->finish;
      # tables_priv - neuvazuji mozne ruzne zaznamy uzivatele pro ruzne Host
      my $sth=$dbh_sys->prepare(q{
         select
            tables_priv.Db
         from
            mysql.tables_priv
         where
            tables_priv.User=?
      });
      $sth->execute($pkg->get_remote_user);
      while(my $ref=$sth->fetchrow_arrayref) {
         push @databases, $$ref[0] if(!grep(/^\Q$$ref[0]\E$/, @databases));
      }
      $sth->finish;
   }
   # pokud uzivatel nema vubec zadna prava
   else {
      die("You have not rights!");
   }
   return sort @databases;
}

#-------------- Funkce zjistujici existenci databaze (pokud uzivatel na ni nema pravo, je to jakoby neexistovala)
sub database_exist {
   my ($pkg_p, $database)=@_;
   my @databases=$pkg->databases;
   return grep(/^\Q$database\E$/,@databases);
}

#-- Funkce vracejici pole se seznamem tabulek na ktere ma dany uzivatel nejake pravo
sub tables {
   my ($pkg_p, $database)=@_;
   my @tables;
   if(length($database)) { ### fce by nemela byt volana bez $database - zkus podminku vyhodit az opravis vsechna volani teto fce
      #my @tables=$dbh_usr->tables();
      my $sth=$dbh_usr->prepare(q{
         show
            tables
         from
            } . $database . q{
      });
      $sth->execute;
      while(my @ref=$sth->fetchrow_array) {
         push(@tables,$ref[0]);
      }
      $sth->finish;
   }
   return sort @tables;
}

#-------------- Funkce zjistujici existenci tabulky (pokud uzivatel na ni nema pravo, je to jakoby neexistovala)
sub table_exist {
   my ($pkg_p,$database,$table)=@_;
   my @tables=$pkg->tables($database,$table);
   return grep(/^\Q$table\E$/,@tables);
}

#-------------- Funkce zjistujici vracejici help pro Polozky, Tabulky a Databaze
sub database_help {
   my ($pkg_p,$database,$table,$field)=@_;
   my @values;
   my $sth=$dbh_sys->prepare(q{
      select
         Database_help.help
      from
         Database_help
      where
         Database_help.database_name=? and
         Database_help.table_name=? and
         Database_help.field_name=?
   });
   $sth->execute($database,"$table","$field");
   my $ref=$sth->fetchrow_hashref;
   my $help_text=$$ref{help};
   $sth->finish;
   return $help_text;
}

return 1;

